using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Elements.Diagnostic
{
    public partial class DiagnosticModule : ElementControl
    {    
        protected volatile bool paused = false;
        IPort diagnosticInfoPort;
        public DiagnosticModule() { InitializeComponent(); }

        protected delegate void DisplayInfoDelegate(string info);
        public virtual void DisplayInfo(string info)
        {
            if (this.InvokeRequired) { BeginInvoke(new DisplayInfoDelegate(DisplayInfo), new object[1] { info }); }
            else 
            {
                if (!paused) { diagTextBox.AppendText(info + (char)(17)); }
            }
        }

        protected override void InitElement()
        {
            base.InitElement();
            diagnosticInfoPort = new Port(this, "DiagnosticInfoPort"); ports.Add(diagnosticInfoPort);
            OperationRessourceHandler m = new OperationRessourceHandler("DisplayMessage", this);
            m.RessourceName = "*";
            m.Enqueue = true;
            diagnosticInfoPort.AddRessourceHandler(m);
        }

        protected virtual void DisplayMessage(IMessage m)
        {
            DisplayInfo(
                "Sender: " + m.Sender.FullURL + "\n" +
                "Target: " + m.Target.FullURL + "\n");

        }

        private void startButton_Click(object sender, EventArgs e)
        {
            paused = false;
            startButton.BackColor = Color.Gold;
            pauseButton.BackColor = this.BackColor;
        }

        private void pauseButton_Click(object sender, EventArgs e)
        {
            paused = true;
            pauseButton.BackColor = Color.Gold;
            startButton.BackColor = this.BackColor;
        }
    }
}