using System;
using System.Collections.Generic;
using System.Text;

namespace Elements
{
    public class Port: IPort
    {
        protected bool disposed = false;
        protected IElement owner;
        protected URL url;
        protected string portName = "AnonymousPort";
        protected List<IResponse> listeners = new List<IResponse>(), listening = new List<IResponse>();
        protected List<IPort> connections = new List<IPort>();
        protected List<RessourceHandler> ressourceHandlers = new List<RessourceHandler>();
        
        public Port(IElement owner, string name) 
        { 
            this.owner = owner; this.portName = name;
            url = new URL(owner.ModuleName, name, "*", this);
        }

        public virtual void Dispose() { Dispose(true); }
        protected virtual void Dispose(bool dispose)
        {
            if (!disposed)
            {
                if (dispose)
                {
                    disposed = true; GC.SuppressFinalize(this);
                    //while (listening.Count != 0) { Quiet(listening[0]); }
                }
            }
        }

        public string PortName { get { return portName; } }
        public URL URL { get { return url; } }
        public List<IResponse> Listeners { get { return listeners; } }

        public virtual void AddRessourceHandler(RessourceHandler resh) { ressourceHandlers.Add(resh); }
        public virtual void RemoveRessourceHandler(RessourceHandler resh) { ressourceHandlers.Remove(resh); }
        public virtual void Listen(IPort p, string listenedRessource, string targetRessource) 
        {
            IResponse resp = new Response(
                new URL(p.URL.Module, p.URL.Port, listenedRessource, p),
                new URL(url.Module, url.Port, targetRessource, this),
                true);
            listening.Add(resp);
            p.Listeners.Add(resp);
        }
        public virtual void Quiet(IPort p, string listenedRessource, string targetRessource) 
        {
            
            foreach (IResponse r in listening)
            {
                if (r.Sender.Device.Equals(p)
                    && r.Sender.Ressource.Equals(listenedRessource)
                    && r.Target.Ressource.Equals(targetRessource))
                {
                    p.Listeners.Remove(r);
                    listening.Remove(r);
                    break;
                }
            }
        }
        public virtual void Connect(IPort p) { connections.Add(p); }
        public virtual void Disconnect(IPort p) { connections.Remove(p); }
        public virtual void AdvertiseRessource(string ressource, object[] data)
        {
            foreach (IResponse listener in listeners)
            {
                if (listener.Sender.Ressource.Equals(ressource))
                {
                    listener.Data = data;
                    listener.Success = true;
                    listener.Target.Device.HandleResponse(listener);
                    //listener.Data = null;
                }
            }
        }


        public virtual void HandleRequest(IRequest r) { PreProcessMessage(r); }
        public virtual void SendRequest(IRequest r) { r.Target.Device.HandleRequest(r); }
        public virtual void SendRequestToAll(string sendingRessource,string targetRessource, object[] args, Verb verb )
        {
            foreach (IPort p in connections)
            {
                URL target = p.URL, sender = this.url;
                target.Ressource = targetRessource;
                sender.Ressource = sendingRessource;
                SendRequest(new Request(target, args,sender, verb));
            }
        }
        public virtual void HandleResponse(IResponse r) { PreProcessMessage(r); }
        public virtual void SendResponse(IResponse r) { r.Target.Device.HandleResponse(r); }
        public virtual void SendResponse(IRequest request, object[] data, bool success)
        {
            SendResponse( new Response(request.Target, request.Sender, data, true));
        }

        protected virtual void PreProcessMessage(IMessage message)
        {
            foreach (RessourceHandler resh in ressourceHandlers)
            {
                if (resh.RessourceName == message.Target.Ressource)
                {
                    owner.HandleRessource(resh, message);
                }
            }
        }
        public override string ToString() { return portName; }
    }
}
