using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using XYZDataSetVisualizer;
using DataSetVisualizer;
using Elements;
using System.Threading;

namespace XYZDataSetVisualizer.GUI
{
    public partial class XYZDataSetViewGUI : UserControl
    {
        protected bool inhibit = false;
        protected Color minusColor = Color.Red, zeroColor = Color.Blue, plusColor = Color.Green;
        protected ColorMode colorMode = ColorMode.SINGLE;
        protected DisplayStyle displayStyle = DisplayStyle.WIREFRAME;
        protected ColorGradient colorGradient = new ColorGradient(
            -Visualizer.NOMINAL_LENGTH / 4,
            Visualizer.NOMINAL_LENGTH / 4,
            Color.Red.ToArgb(),
            Color.Green.ToArgb(),
            Color.Red.ToArgb(),Color.Blue.ToArgb(),Color.Green.ToArgb());

        protected float xDisplacement = 0, yDisplacement = 0, zDisplacement = 0;
        protected float xScaling = 1.0f, yScaling = 1.0f, zScaling = 1.0f;

        protected bool freezed = false;
        protected XYZDataSetView owner;
        protected DisplayModes xDisplayMode = DisplayModes.NONE,
            yDisplayMode = DisplayModes.NONE,
            zDisplayMode = DisplayModes.NONE;

        private struct AxisBoxItem
        {
            public URL item;
            public string pre;
            public AxisBoxItem(string pre, URL item) { this.pre = pre; this.item = item; }
            public override string ToString() { return pre + item.Ressource; }
        }

        public XYZDataSetViewGUI(string name, XYZDataSetView owner) 
        { 
            InitializeComponent();
            dataSetGroupBox.Text = name;
            styleListBox.Items.AddRange(
                new object[] { 
                    DisplayStyle.WIREFRAME, 
                    DisplayStyle.POINTS, 
                    DisplayStyle.SOLID, 
                    DisplayStyle.BARGRAPH });
            singleColorButton.Checked = true;
            minusColorBox.BackColor = minusColor;
            zeroColorBox.BackColor = zeroColor;
            plusColorBox.BackColor = plusColor;
            ColorMode = colorMode;
            Owner = owner;
        }

        public XYZDataSetView Owner
        {
            get { return owner; }
            set
            {
                if (owner != null) { owner.RessourceChanged -= new RessourceChangedHandler(OwnerRessourceChanged); }
                owner = value;
                owner.RessourceChanged += new RessourceChangedHandler(OwnerRessourceChanged);
                foreach (RessourceHandler r in owner.RessourceHandlers)
                {
                    OwnerRessourceChanged(owner, r.RessourceName);
                }
            }
        }

        protected void OwnerRessourceChanged(IElement sender, string ressource)
        {
            switch (ressource)
            {
                case "XDisplacement": break;
                case "DataSources":
                    xAxisListBox.Items.Clear(); 
                    yAxisListBox.Items.Clear(); 
                    zAxisListBox.Items.Clear(); 
                    xAxisListBox.Items.AddRange(new object[3] { DisplayModes.NONE, DisplayModes.INCREMENT, DisplayModes.HISTORY });
                    yAxisListBox.Items.AddRange(new object[3] { DisplayModes.NONE, DisplayModes.INCREMENT, DisplayModes.HISTORY });
                    zAxisListBox.Items.AddRange(new object[3] { DisplayModes.NONE, DisplayModes.INCREMENT, DisplayModes.HISTORY });
                    for(int i=0; i< owner.DataSources.Count; i++)
                    {
                        xAxisListBox.Items.Add(new AxisBoxItem("",owner.DataSources[i]));
                        xAxisListBox.Items.Add(new AxisBoxItem("(Master)", owner.DataSources[i]));
                        yAxisListBox.Items.Add(new AxisBoxItem("", owner.DataSources[i]));
                        yAxisListBox.Items.Add(new AxisBoxItem("(Master)", owner.DataSources[i]));
                        zAxisListBox.Items.Add(new AxisBoxItem("", owner.DataSources[i]));
                        zAxisListBox.Items.Add(new AxisBoxItem("(Master)", owner.DataSources[i]));
                    }
                    OwnerRessourceChanged(sender, "XDataSource");
                    OwnerRessourceChanged(sender, "YDataSource");
                    OwnerRessourceChanged(sender, "ZDataSource");
                    break;
                case "XDataSource":
                    if (owner.XDisplayMode == DisplayModes.SAMPLE_MASTER)
                    {
                        xAxisListBox.SelectedItem = new AxisBoxItem("(Master)", owner.XDataSource);
                    }
                    else if (owner.XDisplayMode == DisplayModes.SAMPLE_SLAVE)
                    {
                        xAxisListBox.SelectedItem = new AxisBoxItem("", owner.XDataSource);
                    }
                    else { xAxisListBox.SelectedItem = owner.XDisplayMode; }
                    break;
                case "YDataSource":
                    if (owner.YDisplayMode == DisplayModes.SAMPLE_MASTER)
                    {
                        yAxisListBox.SelectedItem = new AxisBoxItem("(Master)", owner.YDataSource);
                    }
                    else if (owner.XDisplayMode == DisplayModes.SAMPLE_SLAVE)
                    {
                        yAxisListBox.SelectedItem = new AxisBoxItem("", owner.YDataSource);
                    }
                    else { yAxisListBox.SelectedItem = owner.YDisplayMode; }
                    break;
                case "ZDataSource":
                    if (owner.ZDisplayMode == DisplayModes.SAMPLE_MASTER)
                    {
                        zAxisListBox.SelectedItem = new AxisBoxItem("(Master)", owner.ZDataSource);
                    }
                    else if (owner.XDisplayMode == DisplayModes.SAMPLE_SLAVE)
                    {
                        zAxisListBox.SelectedItem = new AxisBoxItem("", owner.ZDataSource);
                    }
                    else { zAxisListBox.SelectedItem = owner.ZDisplayMode; }
                    break;
                case "ColorMode": ColorMode = owner.ColorMode; break;
                case "SingleColor": SingleColor = owner.SingleColor; break;
                case "ColorGradient": ColorGradient = owner.ColorGradient; break;
                case "DisplayStyle": styleListBox.SelectedItem = owner.DisplayStyle; break;
            }
        }

        
        public virtual ColorMode ColorMode
        {
            get { return colorMode; }
            set 
            {
                switch (value)
                {
                    case ColorMode.SINGLE:
                        plusColorBox.Hide(); minusColorBox.Hide();
                        plusColorLabel.Hide(); minusColorLabel.Hide();
                        zeroColorLabel.Text = "Color";
                        singleColorButton.Checked = true;
                        gradientColorButton.Checked = false;
                        break;
                    case ColorMode.GRADIENT:
                        plusColorBox.Show(); minusColorBox.Show();
                        plusColorLabel.Show(); minusColorLabel.Show();
                        zeroColorLabel.Text = "Color0";
                        singleColorButton.Checked = false;
                        gradientColorButton.Checked = true;
                        break;
                }
                colorMode = value; 
            }
        }

        public virtual Color SingleColor
        {
            get { return zeroColor; }
            set { zeroColor = value; zeroColorBox.BackColor = value; }
        }

        public virtual ColorGradient ColorGradient
        {
            get { return colorGradient; }
            set 
            { 
                colorGradient = value;
                minusColor = Color.FromArgb(value[1]); 
                minusColorBox.BackColor = minusColor;
                zeroColor = Color.FromArgb(value[2]); 
                zeroColorBox.BackColor = zeroColor;
                plusColor = Color.FromArgb(value[3]); 
                plusColorBox.BackColor = plusColor;
            }
        }

        public virtual DisplayStyle DisplayStyle
        {
            get { return displayStyle; }
            set { displayStyle = value; }
        }

        private void plusColorBox_Click(object sender, EventArgs e)
        {
            colorDialog.ShowDialog(FindForm());
            plusColor = colorDialog.Color;
            SetColors();
        }

        private void zeroColorBox_Click(object sender, EventArgs e)
        {
            colorDialog.ShowDialog(FindForm());
            zeroColor = colorDialog.Color;
            SetColors();
            owner.SingleColor = zeroColor;
        }

        private void minusColorBox_Click(object sender, EventArgs e)
        {
            colorDialog.ShowDialog(FindForm());
            minusColor = colorDialog.Color;
            SetColors();
        }

        protected virtual void SetColors()
        {
            //Set single color and colorMode
            colorGradient[1] = minusColor.ToArgb();
            colorGradient[2] = zeroColor.ToArgb();
            colorGradient[3] = plusColor.ToArgb();
            owner.ColorGradient = colorGradient;
        }
        bool extendedVisisble = false;
        private void extendControlButton_Click(object sender, EventArgs e)
        {
            this.SuspendLayout();
            if (!extendedVisisble) 
            { 
                dataSetGroupBox.Width += controlContainer.Panel2.Width;
                extendControlButton.Text = "<";
                extendedVisisble = true;
            }
            else 
            { 
                dataSetGroupBox.Width -= controlContainer.Panel2.Width;
                extendControlButton.Text = ">";
                extendedVisisble = false;
            }
            this.ResumeLayout(); 
        }

        private void gradientColorButton_Click(object sender, EventArgs e)
        {
            owner.ColorMode = ColorMode.GRADIENT;
        }

        private void singleColorButton_Click(object sender, EventArgs e)
        {
            owner.ColorMode = ColorMode.SINGLE;
        }

        private void styleListBox_Click(object sender, EventArgs e)
        {
            DisplayStyle disp;
            switch (styleListBox.SelectedIndex)
            {
                case 0: disp = DisplayStyle.WIREFRAME; break;
                case 1: disp = DisplayStyle.POINTS; break;
                case 2: disp = DisplayStyle.SOLID; break;
                case 3: disp = DisplayStyle.BARGRAPH; break;
                default: disp = DisplayStyle.WIREFRAME; break;
            }
            owner.DisplayStyle = disp;
        }
        private void xAxisListBox_Click(object sender, EventArgs e)
        {
            switch (xAxisListBox.SelectedIndex)
            {
                case 0: owner.XDisplayMode = DisplayModes.NONE; break;
                case 1: owner.XDisplayMode = DisplayModes.INCREMENT; break;
                case 2: owner.XDisplayMode = DisplayModes.HISTORY; break;
                default:
                    if ((xAxisListBox.SelectedIndex - 3) % 2 == 1)
                    {
                        owner.XDisplayMode = DisplayModes.SAMPLE_MASTER;
                    }
                    else { owner.XDisplayMode = DisplayModes.SAMPLE_SLAVE; } 
                    owner.XDataSource = owner.DataSources[(xAxisListBox.SelectedIndex - 3)/2];
                    break;
            }
        }
        private void yAxisListBox_Click(object sender, EventArgs e)
        {
            switch (yAxisListBox.SelectedIndex)
            {
                case 0: owner.YDisplayMode = DisplayModes.NONE; break;
                case 1: owner.YDisplayMode = DisplayModes.INCREMENT; break;
                case 2: owner.YDisplayMode = DisplayModes.HISTORY; break;
                default:
                    if ((yAxisListBox.SelectedIndex - 3) % 2 == 1)
                    {
                        owner.YDisplayMode = DisplayModes.SAMPLE_MASTER;
                    }
                    else { owner.YDisplayMode = DisplayModes.SAMPLE_SLAVE; } 
                    owner.YDataSource = owner.DataSources[(yAxisListBox.SelectedIndex - 3)/2];
                    break;
            }
        }
        private void zAxisListBox_Click(object sender, EventArgs e)
        {
            switch (zAxisListBox.SelectedIndex)
            {
                case 0: owner.ZDisplayMode = DisplayModes.NONE; break;
                case 1: owner.ZDisplayMode = DisplayModes.INCREMENT; break;
                case 2: owner.ZDisplayMode = DisplayModes.HISTORY; break;
                default:
                    if ((zAxisListBox.SelectedIndex - 3) % 2 == 1)
                    {
                        owner.ZDisplayMode = DisplayModes.SAMPLE_MASTER;
                    }
                    else { owner.ZDisplayMode = DisplayModes.SAMPLE_SLAVE; } 
                    owner.ZDataSource = owner.DataSources[(zAxisListBox.SelectedIndex - 3)/2];
                    break;
            }
        }

        private void xPlusButton_Click(object sender, EventArgs e) { owner.XDisplacement += 10; }
        private void xMinusButton_Click(object sender, EventArgs e) { owner.XDisplacement -= 10; }
        private void yPlusButton_Click(object sender, EventArgs e) { owner.YDisplacement += 10; }
        private void yMinusButton_Click(object sender, EventArgs e) { owner.YDisplacement -= 10; }
        private void zPlusButton_Click(object sender, EventArgs e) { owner.ZDisplacement += 10; }
        private void zMinusButton_Click(object sender, EventArgs e) { owner.ZDisplacement -= 10; }

        private void xScaleBar_Scroll(object sender, EventArgs e)
        {
            float scale = 1.0f;
            if (xScaleBar.Value > 0) { scale += xScaleBar.Value/4f; }
            else if (xScaleBar.Value == 0) { scale = 1.0f; }
            else if (xScaleBar.Value < 0) { scale = 1.0f / Math.Abs(xScaleBar.Value*4f); }

            owner.XScaling = scale;
        }

        private void yScaleBar_Scroll(object sender, EventArgs e)
        {
            float scale = 1.0f;
            if(yScaleBar.Value>0){ scale += yScaleBar.Value/4f; }
            else if(yScaleBar.Value==0){ scale = 1.0f; }
            else if (yScaleBar.Value < 0) { scale = 1.0f / Math.Abs(yScaleBar.Value * 4f); }

            owner.YScaling = scale;
        }

        private void zScaleBar_Scroll(object sender, EventArgs e)
        {
            float scale = 1.0f;
            if (zScaleBar.Value > 0) { scale += zScaleBar.Value / 4f; }
            else if (zScaleBar.Value == 0) { scale = 1.0f; }
            else if (zScaleBar.Value < 0) { scale = 1.0f / Math.Abs(zScaleBar.Value * 4f); }

            owner.ZScaling = scale;
        }

        private void forwardButton_Click(object sender, EventArgs e)
        {
            owner.Forward();
        }


        private void freezeButton_Click(object sender, EventArgs e)
        {
            if (!freezed)
            {
                owner.Freeze();
                freezeButton.Text = "Unfreeze";
                freezed = true;
            }
            else
            {
                owner.Unfreeze();
                freezeButton.Text = "Freeze";
                freezed = false;
            }
        }

        private void rewindButton_Click(object sender, EventArgs e)
        {
            owner.Rewind();
        }

        

    }
}
