using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace DataSetVisualizer
{
    public class ColorGradient
    {
        protected struct ColorValuePair { public byte[] color; public float value; }
        protected List<ColorValuePair> colorList = new List<ColorValuePair>();

        public ColorGradient(int lowColor, int highColor) { InitializeColorList(lowColor, highColor); }
        public ColorGradient(float rangeMinus, float rangePlus, int lowColor, int highColor, params int[] colors)
        {
            float increment = (rangePlus - rangeMinus)/(colors.Length-1);
            
            InitializeColorList(lowColor, highColor);

            for (int i = 0; i < colors.Length; i++)
            {
                AddColorValue(colors[i], rangeMinus + increment * i);
            }
        }

        protected virtual void InitializeColorList(int lowColor, int highColor)
        {
            ColorValuePair cvpl, cvph;

            cvpl.color = BitConverter.GetBytes(lowColor); cvpl.value = float.MinValue;
            cvph.color = BitConverter.GetBytes(highColor); cvph.value = float.MaxValue;
            colorList.Add(cvpl); colorList.Add(cvph);
        }

        public virtual int this[int i] 
        {
            get { return BitConverter.ToInt32(colorList[i].color, 0) ; } 
            set 
            {
                ColorValuePair p;
                p.color = BitConverter.GetBytes(value);
                p.value = colorList[i].value;
                colorList[i] = p; 
            } 
        }

        public int GetColorForValue(float value)
        {
            byte a = 0, r = 0, g = 0, b = 0;
            int color;

            for (int i = 1; i < colorList.Count; i++)
            {
                if (value < colorList[i].value)
                {
                    float ratio = (value + Math.Abs(colorList[i - 1].value)) / (Math.Abs(colorList[i - 1].value) + Math.Abs(colorList[i].value));
                    a = (byte)(colorList[i - 1].color[0] + (colorList[i].color[0] - colorList[i - 1].color[0]) * ratio);
                    r = (byte)(colorList[i - 1].color[1] + (colorList[i].color[1] - colorList[i - 1].color[1]) * ratio);
                    g = (byte)(colorList[i - 1].color[2] + (colorList[i].color[2] - colorList[i - 1].color[2]) * ratio);
                    b = (byte)(colorList[i - 1].color[3] + (colorList[i].color[3] - colorList[i - 1].color[3]) * ratio);

                    break;
                }
            }

            /*color = (int)a << 24;
            color += (int)r << 16;
            color += (int)g << 8;
            color += b;*/
            color = BitConverter.ToInt32(new byte[4] { a, r, g, b },0);

            return color;
        }
        public int[] GetColorForValues(float[] values)
        {
            int[] colorArray = new int[values.Length];
            for (int i = 0; i < values.Length; i++) { colorArray[i] = GetColorForValue(values[i]); }
            return colorArray;
        }

        //public int GetColorAtIndex(int index) { return BitConverter.ToInt32(colorList[index].color, 0); }
        public float GetValueAtIndex(int index) { return colorList[index].value; }

        public void AddColorValue(int color, float value)
        {
            for (int i = 0; i < colorList.Count; i++)
            {
                if (value <= colorList[i].value)
                {
                    ColorValuePair cvp; cvp.value = value; cvp.color = BitConverter.GetBytes(color);
                    colorList.Insert(i, cvp);
                    break;
                }
            }
        }
        public void RemoveColorValue(float value)
        {
            for (int i = 1; i < colorList.Count - 1; i++) { if (colorList[i].value == value) { colorList.RemoveAt(i); break; } }
        }

    }

}
