using System;
using System.Collections.Generic;
using System.Text;
using Motr3D;
using Math32.LinearAlgebra;

namespace DataSetVisualizer
{
    public class HistoryAxis: Axis
    {
        private ProportionalAxisGraduation graduation;
        
        public HistoryAxis(Vertex start, Vertex end, Vertex normal, Object3D dataSet, string axisName)
            : base(start, end,normal, dataSet, axisName)
        {
            Units = "Samples";
            graduation = new ProportionalAxisGraduation(start, end, normal, maxDepthLength / 5 , 15, true);
            graduation.LowerValue = 0;
            graduation.UpperValue = maxDepthLength;
            graduation.UpperBound = direction * maxDepthLength;
            AddChild(graduation);
            Scaling = 1.0f;
            //Scaling = Visualizer.NOMINAL_LENGTH / 100.0f;
        }
        
        public override int Dimensions { get { return 2; } }

        public override float Scaling
        {
            get { return base.Scaling / 16f;}
            set
            {
                base.Scaling = value * 16f;
                graduation.UpperBound = start + direction * (maxDepthLength * (value * 16f));
            }
        }

        public override float Displacement
        {
            get { return base.Displacement; }
            set
            {
                base.Displacement = value;
                graduation.ResetPosition();
                graduation.Translate(Vertex.Normalize(End - Start) * value);
            }
        }

        public override float[][] GetValues(int position)
        {
            int pos = 0, depth = valuesHistory.Count;

            if (position + maxDepthLength > maxHistoryLength) 
            {
                pos = position; 
                depth = maxHistoryLength - position;
            }
            if (depth > maxDepthLength) { depth = maxDepthLength; }

            return valuesHistory.GetRange(pos, depth).ToArray();
        }
        
        public override void CreateValues(int length)
        {
            
            if (valuesHistory.Count < maxHistoryLength)
            {
                valuesHistory.Add(CreateFillingValues(length, valuesHistory.Count));
            }
                    
               
            if (valuesHistory.Count > maxHistoryLength)
            {
                valuesHistory.RemoveRange(maxHistoryLength, valuesHistory.Count - maxHistoryLength);
            }
            
        }
        protected virtual float[] CreateFillingValues(int length, float filler)
        {
            float[] values = new float[length];
            for (uint i = 0; i < length; i++) { values[i] = filler; }
            return values;
        }
    }
}
