using System;
using System.Collections.Generic;
using System.Text;
using Motr3D;
using DataSetTypes;
using Math32.LinearAlgebra;

namespace DataSetVisualizer
{
    public class IncrementAxis: Axis
    {
        protected string units = "()";
        protected AxisGraduation graduations;
        
        public IncrementAxis(Vertex start, Vertex end, Vertex normal, Object3D dataSet, string axisName)
            : base(start, end, normal, dataSet, axisName)
        {
            Units = "Increment";
            graduations = new AxisGraduation(start,end, normal, 100, 15, true);
            graduations.LowerValue = 0;
            graduations.UpperValue = 10;
        }

        public override int Dimensions { get { return 1; } }

        public override void CreateValues(int length)
        {
            valuesHistory.Insert(0, CreateIncrementingValues(length));
            
            if (valuesHistory.Count > maxHistoryLength)
            {
                valuesHistory.RemoveRange(maxHistoryLength, valuesHistory.Count - maxHistoryLength);
            }
            
        }
        protected virtual float[] CreateIncrementingValues(int length)
        {
            float[] values = new float[length];
            float val = 0;
            for (uint i = 0; i < length; i++, val++) { values[i] = val; }
            return values;
        }
    }
}
