using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Elements;

namespace Driver
{
    public enum States {STARTED = 1, STOPPED = 2};
    
    public abstract class AbstractDriver : Element
    {
        protected object propertiesLock = new object();
        protected object objectLock = new object();
        protected uint interfaceNumber;
        protected States state = States.STOPPED;

        private RessourceHandler stateResH, startResH, stopResH;

        protected internal AbstractDriver(string name) : base(name) { }

        protected override void InitElement()
        {
            base.InitElement();
            stateResH = new AttributeRessourceHandler("State", controlPort);
            startResH = new OperationRessourceHandler("Start", controlPort);
            stopResH = new OperationRessourceHandler("Stop", controlPort);
        }

        public virtual States State
        {
            get { return state; }
            set { if (value == States.STARTED) { Start(); } else { Stop(); } }
        }

        public virtual void Start() { state = States.STARTED; }
        public virtual void Stop() { state = States.STOPPED; }
    }
}
