using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.ComponentModel.Design;

namespace Driver
{
    public abstract class Enumerator : Component
    {
        protected List<AbstractDriver> dataInterfaces = new List<AbstractDriver>();
        protected bool disposed = false;

        public Enumerator() { FindDevices(); }
        public Enumerator(IContainer container)
        {
            container.Add(this, this.ToString());
            FindDevices();
        }
        ~Enumerator() { Dispose(false); }

        public virtual List<AbstractDriver> Drivers { get { return dataInterfaces; } }

        public override string ToString() { return "Data Interface Server"; }

        protected override void Dispose(bool disposing)
        {
            if (!disposed)
            {
                base.Dispose(disposing);
                if (disposing)
                {
                    disposed = true;
                    foreach (AbstractDriver s in dataInterfaces) { s.Dispose(); }
                }
            }
        }

        public virtual AbstractDriver GetDriver(uint id)
        {
            if (id > (dataInterfaces.Count - 1)) { throw new ArgumentOutOfRangeException(); }
            return (dataInterfaces[(int)id]);
        }

        protected abstract void FindDevices();
    }
}
