using System;
using System.Collections.Generic;
using System.Text;

namespace Elements
{
    public enum Verb { GET = 1, POST = 2, PUT = 3 };
    public interface IMessage { URL Sender { get;} URL Target { get;} }
    public interface IRequest: IMessage { object[] Args { get;} Verb Verb { get;} }
    public struct Request: IRequest
    {
        private object[] args;
        private URL sender, target;
        private Verb verb;
        public Request(URL target, object[] args, URL sender, Verb verb) 
        {
            this.target = target; this.args = args; this.sender = sender; this.verb = verb;
        }
        public URL Target { get { return target; } }
        public object[] Args { get { return args; } }
        public URL Sender { get { return sender; } }
        public Verb Verb { get { return verb; } }
    }

    public interface IResponse : IMessage { object[] Data { get; set;} bool Success { get; set; } }
    public struct Response : IResponse
    {
        private object[] data;
        private bool success;
        private URL sender, target;
        public Response( URL sender, URL target, object[] data, bool success)
        {
            this.data = data; this.success = success; this.sender = sender; this.target = target;
        }
        public Response(URL sender, URL target, bool success) 
        {
            this.data = null; this.success = success; this.sender = sender; this.target = target;
        }

        public object[] Data { get { return data; } set { data = value; } }
        public bool Success { get { return Success; } set { success = value; } }
        public URL Sender { get { return sender; } }
        public URL Target { get { return target; } }
    }

    /*
     *URL Format: (location and interface secific)/Module/Port/Ressource
     * 
     */
    public struct URL
    {
        private string url;
        private string element, port, ressource;
        private IPort device;
        private bool isValid;

        public URL(string url, IPort device) 
        {
            this.url = ""; this.element = ""; this.port = ""; this.ressource = ""; this.device = device;
            isValid = true;
            AssembleURL(url);
        }
        public URL(string module, string port, string ressource, IPort device) 
        {
            this.url = ""; this.element = module; this.port = port; this.ressource = ressource; this.device = device;
            isValid = true;
            AssembleURL();
        }

        private void AssembleURL() {  url = "/" + element + "/" + port + "/" + ressource; }
        private void AssembleURL(string url)
        {
            this.url = url;
            element = url.Split(new string[1] { "/" }, 3, StringSplitOptions.None)[0];
            port = url.Split(new string[1] { "/" }, 3, StringSplitOptions.None)[1];
            ressource = url.Split(new string[1] { "/" }, 3, StringSplitOptions.None)[2];
        }

        public string FullURL { get { return url; } set { AssembleURL(value); } }
        public string Module { get { return element; } set { element = value; AssembleURL(); } }
        public string Port { get { return port; } set { port = value; AssembleURL(); } }
        public string Ressource { get { return ressource; } set { ressource = value; AssembleURL(); } }
        public IPort Device { get { return device; } set { device = value; } }
        public bool IsValid { get { return isValid; } set { isValid = value; } }

        public override bool Equals(object obj)
        {
            if (obj is URL)
            {
                URL url = (URL)obj;
                
                if (this.url != url.FullURL) { return false; }
                if (!device.Equals(url.Device)) { return false; }
                return true;
            }
            return false;
        }
    }
}
