using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Elements;

namespace SignalProcessing.GUI
{
    public partial class PerformanceFilterGUI : SignalProcessorGUI
    {
        protected bool inhibit = false;
        private PerformanceFilter owner;

        public PerformanceFilterGUI(PerformanceFilter owner): base(owner)
        { 
            InitializeComponent();
            Owner = owner;
        }

        public PerformanceFilter Owner
        {
            get { return owner; }
            set
            {
                if (owner != null) { owner.RessourceChanged -= new RessourceChangedHandler(OwnerRessourceChanged); }
                owner = value;
                owner.RessourceChanged += new RessourceChangedHandler(OwnerRessourceChanged);
                foreach (RessourceHandler r in owner.RessourceHandlers)
                {
                    OwnerRessourceChanged(owner, r.RessourceName);
                }
            }
        }

        protected override void OwnerRessourceChanged(IElement sender, string ressource)
        {
            inhibit = true;
            switch (ressource)
            {
                case "Skip": skipIntervalUpDown.Value = owner.Skip; break;
                case "Decimate": decimateIntervalUpDown.Value = owner.Decimate; break;
            }
            inhibit = false;
        }
       
        private void decimateIntervalUpDown_ValueChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.Decimate = (uint)decimateIntervalUpDown.Value;
            
        }

        private void skipIntervalUpDown_ValueChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.Skip = (uint)skipIntervalUpDown.Value;
        }
    }
}
