using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Elements;
using SignalProcessing;

namespace SignalProcessing.GUI
{
    public partial class TriggerGUI : SignalProcessorGUI
    {
        protected bool inhibit = false, isRisingEdge = true, enable = true;
        protected int levelRatio = 50, preTriggerRatio = 0, postTriggerRatio = 100;
        private Trigger owner;

        public TriggerGUI(Trigger owner) : base(owner) { InitializeComponent(); Owner = owner; }

        public Trigger Owner
        {
            get { return owner; }
            set
            {
                if (owner != null) { owner.RessourceChanged -= new RessourceChangedHandler(OwnerRessourceChanged); }
                owner = value;
                owner.RessourceChanged += new RessourceChangedHandler(OwnerRessourceChanged);
                foreach (RessourceHandler r in owner.RessourceHandlers)
                {
                    OwnerRessourceChanged(owner, r.RessourceName);
                }
            }
        }

        protected override void OwnerRessourceChanged(IElement sender, string ressource)
        {
            inhibit = true;
            switch (ressource)
            {
                case "IsRisingEdge":
                    risingRadioButton.Checked = owner.IsRisingEdge;
                    fallingRadioButton.Checked = !owner.IsRisingEdge; 
                    break;
                case "LevelRatio":
                    levelNumericUpDown.Value = (int)(owner.LevelRatio * 100);
                    levelTrackBar.Value = (int)(owner.LevelRatio * 100); 
                    break;
                case "PostTriggerRatio":
                    postTriggerNumericUpDown.Value = (int)(owner.PostTriggerRatio * 100);
                    postTriggerTrackBar.Value = (int)(owner.PostTriggerRatio * 100);
                    break;
                case "PreTriggerRatio":
                    preTriggerNumericUpDown.Value = (int)(owner.PreTriggerRatio * 100);
                    preTriggerTrackBar.Value = (int)(owner.PreTriggerRatio * 100);
                    break;
                case "Enable": enableCheckBox.Checked = owner.Enable; break;


            }
            inhibit = false;
        }

        private void risingRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.IsRisingEdge = risingRadioButton.Checked;
        }

        private void fallingRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.IsRisingEdge = !fallingRadioButton.Checked;
           
        }

        private void levelNumericUpDown_ValueChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.LevelRatio = (float)levelNumericUpDown.Value / 100.0f; 
        }

        private void levelTrackBar_ValueChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.LevelRatio = (float)levelTrackBar.Value / 100.0f;  
        }

        private void enableCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.Enable = enableCheckBox.Checked;
        }

        private void preTriggerNumericUpDown_ValueChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.PreTriggerRatio = (float)preTriggerNumericUpDown.Value / 100.0f;
            
        }

        private void postTriggerNumericUpDown_ValueChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.PostTriggerRatio = (float)postTriggerNumericUpDown.Value / 100.0f;
        }

        private void preTriggerTrackBar_ValueChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.PreTriggerRatio = (float)preTriggerTrackBar.Value / 100.0f;
        }

        private void postTriggerTrackBar_ValueChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.PostTriggerRatio = (float)postTriggerTrackBar.Value / 100.0f;
        } 
    }
}
