using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using DataSetTypes;
using System.Windows.Forms;
using Elements;

namespace SignalProcessing
{
    public class PerformanceFilter : SignalProcessor
    {
        public const uint DEFAULT_DECIMATE = 1;
        public const uint DEFAULT_SKIP = 0;

        protected volatile uint decimate = DEFAULT_DECIMATE, skip = DEFAULT_SKIP, skipped = 0;

        private RessourceHandler decimateResH, skipResH;

        public PerformanceFilter(string name) : base(name) {  }

        protected override void InitElement()
        {
            base.InitElement();
            decimateResH = new AttributeRessourceHandler("Decimate", controlPort);
            skipResH = new AttributeRessourceHandler("Skip", controlPort);
        }

        public virtual uint Decimate 
        { 
            get { return decimate; }
            set 
            {
                decimate = value;
                decimateResH.Advertise(value);
            } 
        }
        public virtual uint Skip
        {
            get { return skip; }
            set
            {
                skip = value;
                skipResH.Advertise(value);
            }
        }

        public override DataSet ProcessSignal(DataSet signal)
        {
            DataSet filteredSignal;
            uint async_decimate = decimate;

            if (skipped < skip) { skipped++; return null; }
            skipped = 0; ;

            byte[] input = signal.Buffer;
            ushort blockSize = signal.BlockSize;
            uint blocks = (uint)(input.Length / blockSize);
            byte[] output = new byte[(uint)((blocks / async_decimate)*blockSize)];

            for (uint i = 0; i < output.Length; i++)
            {
                for (uint j = 0; j < blockSize; j++) { output[i + j] = input[(i + j) * async_decimate]; }
            }
            return new DataSet(
                output,
                signal.MasterParameter,
                signal.SlaveParameter,
                blockSize);            
        }
    }
}
