using System;
using System.Collections.Generic;
using System.Text;
using DataSetTypes;
using Elements;
using System.Windows.Forms;

namespace SignalProcessing
{
    public abstract class SignalProcessor : Element
    {
        internal object objectLock = new object();
        protected IPort inputSignalPort, outputSignalPort;
        private RessourceHandler outputSignalResH, inputSignalResH;
        protected DataSet inputSignal, outputSignal;
        protected bool invalid = false;

        public SignalProcessor(string name): base(name) { }

        protected override void InitElement()
        {
            base.InitElement();
            inputSignalPort = new Port(this, "DataInput"); ports.Add(inputSignalPort);
            outputSignalPort = new Port(this, "DataOutput"); ports.Add(outputSignalPort);
            inputSignalResH =  new AttributeRessourceHandler("InputSignal", inputSignalPort);
            inputSignalResH.Enqueue = true; inputSignalResH.BlockOnInvoke = true;
            outputSignalResH = new AttributeRessourceHandler("OutputSignal", outputSignalPort);
        }
        public virtual IPort SignalInputPort { get { return inputSignalPort; } }
        public virtual IPort SignalOutputPort { get { return outputSignalPort; } }


        public virtual DataSet InputSignal 
        { 
            get { return inputSignal; }
            set { 
                inputSignal = value; invalid = true;
                outputSignal = ProcessSignal(inputSignal);
                if (outputSignal == null) { return; }
                outputSignalPort.AdvertiseRessource("OutputSignal", outputSignal);
            } 
        }
        public virtual DataSet OutputSignal 
        { 
            get  { return outputSignal; } 
            set { outputSignal = value;  } 
        }

        public abstract DataSet ProcessSignal(DataSet signal);
    }
}
