using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Elements;
using VirtualSignalGeneratorDriver;

namespace VirtualSignalGeneratorDriver.GUI
{
    public partial class VirtualSignalGeneratorGUI : Driver.DriverControl
    {
        private bool inhibit = false;
        private VirtualSignalGenerator owner;
        private float dcOffset, lowRef, upRef;
        
        public VirtualSignalGeneratorGUI( VirtualSignalGenerator owner )
        {
            InitializeComponent();
            Owner = owner;

            typeListBox.Items.AddRange(
                new object[4] { 
                    SignalType.SINE, SignalType.COSINE, 
                    SignalType.SQUARE, SignalType.TRIANGLE});
        }

        public VirtualSignalGenerator Owner
        {
            get { return owner; }
            set 
            {
                if (owner != null) { owner.RessourceChanged += new RessourceChangedHandler(OwnerRessourceChanged); }
                owner = value;
                owner.RessourceChanged += new RessourceChangedHandler(OwnerRessourceChanged);
                foreach (RessourceHandler r in owner.RessourceHandlers)
                {
                    OwnerRessourceChanged(owner, r.RessourceName);
                }
            }
        }
        
        private void OwnerRessourceChanged(IElement sender, string ressource)
        {
            inhibit = true;

            switch (ressource)
            {
                case "LowerReference":
                    amplitudeUpDown.Value = (decimal)(owner.UpperReference - owner.LowerReference);
                    amplitudeTrackBar.Value = (int)amplitudeUpDown.Value*10;
                    break;
                case "UpperReference":
                    amplitudeUpDown.Value = (decimal)(owner.UpperReference - owner.LowerReference);
                    amplitudeTrackBar.Value = (int)amplitudeUpDown.Value*10;
                    break;
                case "SignalType":
                    typeListBox.SelectedItem = owner.SignalType;
                    break;
            }

            inhibit = false;
        }

        private void UpdateReferences()
        {
            owner.LowerReference = lowRef + dcOffset;
            owner.UpperReference = dcOffset + upRef;
        }

        private void dcOffetUpDown_Scroll(object sender, ScrollEventArgs e)
        {
            dcOffset = (float)dcOffetUpDown.Value; UpdateReferences();
        }

        private void dcOffsetTrackBar_Scroll(object sender, EventArgs e)
        {
            dcOffset = dcOffsetTrackBar.Value / 10; UpdateReferences();
        }
        private void amplitudeTrackBar_Scroll(object sender, EventArgs e)
        {
            lowRef = -(amplitudeTrackBar.Value / 0.2f);
            upRef = amplitudeTrackBar.Value / 0.2f;
        }

        private void amplitudeUpDown_Scroll(object sender, ScrollEventArgs e)
        {
            lowRef = -((float)amplitudeUpDown.Value / 2);
            upRef = (float)amplitudeUpDown.Value / 2;
            UpdateReferences();
        }

        private void typeListBox_Click(object sender, EventArgs e)
        {
            owner.SignalType = (SignalType)typeListBox.SelectedItem;
        }
    }
}
