using System;
using System.Collections.Generic;
using System.Text;

namespace VirtualSignalGeneratorDriver
{
    public enum SignalType { NONE, SINE, COSINE, SQUARE, TRIANGLE, SAWTOOTH, DC, RAMP, FILE, MATH, ARRAY }
    
    internal interface ISignalGenerator
    {
        byte[] GenerateSignal(int samples);
    }

    internal struct Sine : ISignalGenerator
    {
        public byte[] GenerateSignal(int samples)
        {
            byte[] signal = new byte[samples];
            double angleStep = (2 * Math.PI) / samples;

            for (int i = 0; i < samples; i++)
            {
               signal[i] = (byte)(Math.Sin(angleStep * i)*(byte.MaxValue/2) + (byte.MaxValue/2));
            }
            return signal;
        }
    }
    internal struct Cosine : ISignalGenerator
    {
        public byte[] GenerateSignal(int samples)
        {
            byte[] signal = new byte[samples];
            double angleStep = (2 * Math.PI) / samples;

            for (int i = 0; i < samples; i++)
            {
                signal[i] = (byte)(Math.Cos(angleStep * i) * (byte.MaxValue / 2) + (byte.MaxValue / 2));
            }
            return signal;
        }
    }

    internal struct Triangle : ISignalGenerator
    {
        public byte[] GenerateSignal(int samples)
        {
            byte[] signal = new byte[samples];
            float increment =  2f *(float)byte.MaxValue / samples;
            
            for (int i = 0; i < samples / 2; i++) { signal[i] = (byte)(increment * (float)i); }
            for (int i = 0; i < samples / 2; i++) 
            { 
                signal[i + samples / 2] = (byte)(byte.MaxValue - increment * i); 
            }
                
            return signal;
        }
    }
    internal struct Square : ISignalGenerator
    {
        public byte[] GenerateSignal(int samples)
        {
            byte[] signal = new byte[samples];

            for (int i = 0; i < samples/2; i++) { signal[i] = byte.MinValue; }
            for (int i = samples/2; i < samples; i++) { signal[i] = byte.MaxValue; }

            return signal;
        }
    }
}
