using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Threading;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Driver;
using Elements;

namespace SoundCardInputDriver.GUI
{
    public partial class DriverTestControlGUI : UserControl
    {
        private SoundInputChannel owner;

        public DriverTestControlGUI(SoundInputChannel owner )
        {
            InitializeComponent();
            Owner = owner;
            owner.RessourceChanged += new RessourceChangedHandler(OwnerRessourceChanged);
        }

        public SoundInputChannel Owner
        {
            get { return owner; }
            set
            {
                if (owner != null) { owner.RessourceChanged -= new RessourceChangedHandler(OwnerRessourceChanged); }
                owner = value;
                owner.RessourceChanged+= new RessourceChangedHandler(OwnerRessourceChanged);
                foreach (RessourceHandler r in owner.RessourceHandlers)
                {
                    OwnerRessourceChanged(owner, r.RessourceName);
                }
            }
        }

        protected virtual void OwnerRessourceChanged(IElement sender, string ressource)
        {
            if (ressource == "Sample") { DisplaySampleInfo(); }
        }
        
        public void DisplaySampleInfo() { UpdateDiagInfo("Sample received at " + DateTime.Now.ToString()); }
        protected delegate void UpdateDiagInfoDelegate(string text);
        protected void UpdateDiagInfo(string text)
        {
            if (this.InvokeRequired)
            {
                UpdateDiagInfoDelegate del = new UpdateDiagInfoDelegate(UpdateDiagInfo);
                BeginInvoke(del, new object[1] { text });
            }
            else { diagInfoTextBox.AppendText(text + "\n"); }
        }

        private void startButton_Click(object sender, EventArgs e) 
        {
            owner.Start();
        }

        private void stopButton_Click(object sender, EventArgs e) 
        {
            owner.Stop();
        }
    }
}
