using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using Driver;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Elements;

namespace SoundCardInputDriver.GUI
{
    public partial class SoundInputChannelGUI : DriverControl
    {
        private volatile bool inhibit = false;
        private SoundInputChannel owner;
        
        public SoundInputChannelGUI(SoundInputChannel owner)
        {
            InitializeComponent();
            rateListBox.Items.AddRange(
                new object[] { Rates.R_11KHz, Rates.R_22KHz, Rates.R_44KHz });//, Rates.R_48KHz, Rates.R_96KHz});

            bpsComboBox.Items.AddRange(new object[] { Bps.B_8Bits });
            Owner = owner;
        }

        public SoundInputChannel Owner
        {
            get { return owner; }
            set
            {
                if (owner != null) { owner.RessourceChanged -= new RessourceChangedHandler(OwnerRessourceChanged); }
                owner = value;
                owner.RessourceChanged += new RessourceChangedHandler(OwnerRessourceChanged);
                foreach (RessourceHandler r in owner.RessourceHandlers)
                {
                    OwnerRessourceChanged(owner, r.RessourceName);
                }
            }
        }

        protected virtual void OwnerRessourceChanged(IElement sender, string ressource)
        {
            inhibit = true;
            switch (ressource)
            {
                case "BufferSize": bufferSizeList.SelectedItem = owner.BufferSize.ToString(); break;
                case "BitsPerSample": bpsComboBox.SelectedItem = owner.BitsPerSample; break;
                case "CurrentRate": rateListBox.SelectedItem = owner.CurrentRate; break;
            }
            inhibit = false;
        }

        private void bpsComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.BitsPerSample = (Bps)bpsComboBox.SelectedItem;
        }
        private void bufferSizeList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.BufferSize = (int)(128 * Math.Pow(2, bufferSizeList.SelectedIndex));
        }

        private void rateListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (inhibit) { return; }
            owner.CurrentRate = (Rates)rateListBox.SelectedItem;
        }

    }
}
