using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Threading;
using Microsoft.DirectX.DirectSound;
using System.ComponentModel.Design;
using Driver;
using System.Windows.Forms;

namespace SoundCardInputDriver
{
    public sealed class SoundInputEnumerator : Enumerator
    {
        private static bool enumerated = false;
        private static object typeLock = new object();
        private List<SoundInputDriver> soundInputDrivers = new List<SoundInputDriver>();

        public SoundInputEnumerator() : base() { }
        public SoundInputEnumerator(IContainer container) : base(container) { }

        public override string ToString() { return "Sound Card Input Enumerator"; }

        protected override void FindDevices()
        {
            lock (typeLock) { if (!enumerated) { enumerated = true; } else { return; } }

            CaptureDevicesCollection captureDevices = new CaptureDevicesCollection();
            if (!captureDevices.MoveNext()) { return; } //There is no sound card on the system

            while(captureDevices.MoveNext())
            {
                try
                {                    
                    DeviceInformation dev = (DeviceInformation)captureDevices.Current;
                    SoundInputDriver sd = new SoundInputDriver(dev);
                    for (uint j = 0; j < (uint)sd.Channels; j++) 
                    {
                        SoundInputChannel sc = new SoundInputChannel(sd, j);
                        dataInterfaces.Add(sc);
                        sd.RegisterSoundInputChannel(sc);
                    }
                    soundInputDrivers.Add(sd);
                }
                catch (NullReferenceException e) { }
            }

            captureDevices.Reset();
        }

        /*[STAThread]
        public static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new Test.DriverTest());
        }*/

    }
}
