using System;
using System.Collections.Generic;
using System.Text;
using Elements;

namespace URLServer
{
    public class Server: Element, IRessourceLocator
    {
        private List<URL> register = new List<URL>();
        private IPort urlRegisterInput, urlRegisterOutput;
        private URL[] lastAddedURLs, lastRemovedURLs;
        private RessourceHandler lastAddedURLsResH,
            lastRemovedURLsResH,
            registerURLsResH,
            unRegisterURLsResH,
            searchRessourceResH,
            urlsRessourceResH;
            

        public Server(string name):base(name) {  }

        protected override void InitElement()
        {
            base.InitElement();
            urlRegisterInput = new Port(this, "URLRegisterInput");
            urlRegisterOutput = new Port(this, "URLRegiterOutput");
            lastAddedURLsResH = new AttributeRessourceHandler("LastAddedURLs", urlRegisterOutput);
            lastRemovedURLsResH = new AttributeRessourceHandler("LastRemovedURLs", urlRegisterOutput);
            searchRessourceResH = new OperationRessourceHandler("SearchRessource", urlRegisterOutput);
            registerURLsResH = new OperationRessourceHandler("RegisterURLs", urlRegisterInput);
            unRegisterURLsResH = new OperationRessourceHandler("UnRegisterURLs", urlRegisterInput);
            urlsRessourceResH = new AttributeRessourceHandler("URLs", urlRegisterOutput);
        }

        public IPort URLRegisterInput { get { return urlRegisterInput; } }
        public IPort URLRegisterOutput { get { return urlRegisterOutput; } }

        public URL[] URLs { get { return register.ToArray(); } }
        public URL[] LastAddedURLs 
        { 
            get { return lastAddedURLs; }
            set 
            { 
                lastAddedURLs = value; 
                urlRegisterOutput.AdvertiseRessource("LastAddedURLs", value); 
            }
        }

        public URL[] LastRemovedURLs
        { 
            get { return lastRemovedURLs; }
            set 
            { 
                lastRemovedURLs = value; 
                urlRegisterOutput.AdvertiseRessource("LastRemovedURLs", value); 
            } 
        }

        public void RegisterURLs(params URL[] urls)
        {
            for (int i = 0; i < urls.Length; i++)
            {
                if (!register.Contains(urls[i])) { register.Add(urls[i]); }
                LastAddedURLs = urls;
            }
        }

        public void UnRegisterURLs(params URL[] urls)
        {
            for (int i = 0; i < urls.Length; i++)
            {
                register.Remove(urls[i]);
                LastRemovedURLs = urls;
            }
        }

        public URL[] SearchRessource(string ressource)
        {
            List<URL> urls = new List<URL>();

            foreach (URL url in register) { if (url.Ressource == ressource) { urls.Add(url); } }

            return urls.ToArray();
        }
    }
}
