using System;
using System.Collections.Generic;
using System.Text;

namespace Math32.LinearAlgebra
{
    public struct Matrix22
    {
        public float[,] m;

        public float this[int i, int j] { get { return m[i, j]; } set { m[i, j] = value; } }

        public static Matrix22 Identity 
        { 
            get 
            {
                Matrix22 m;
                m.m = new float[2, 2] { { 1, 0 }, { 0, 1 } };
                return m;
            } 
        }


        public void Invert() { this = Matrix22.Inverse(this); }
        public static Matrix22 Inverse(Matrix22 mat)
        {
            Matrix22 res;
            float a = mat[0, 0], b = mat[0, 1], c = mat[1, 0], d = mat[1, 1];
            res.m = new float[2, 2] { { d, -b }, { -c, a } };
            res *= 1 / (a * d - b * c);
            return res;   
        }

        public static Matrix22 operator *(Matrix22 a, float f)
        {
            a[0, 0] *= f; a[0, 1] *= f; a[1, 0] *= f; a[1, 1] *= f;
            return a;
        }
        public static Matrix22 operator *(float f, Matrix22 mat)
        {
            mat[0, 0] *= f; mat[0, 1] *= f; mat[1, 0] *= f; mat[1, 1] *= f;
            return mat;
        }

        public static Matrix22 operator +(Matrix22 a, Matrix22 b)
        {
            a[0, 0] += b[0, 0]; a[0, 1] += b[0, 1]; a[1, 0] += b[1, 0]; a[1, 1] += b[1, 1];
            return a;
        }
        public static Matrix22 operator -(Matrix22 a, Matrix22 b)
        {
            a[0, 0] -= b[0, 0]; a[0, 1] -= b[0, 1]; a[1, 0] -= b[1, 0]; a[1, 1] -= b[1, 1];
            return a;
        }

        public static Matrix22 operator -(Matrix22 mat)
        {
            mat[0, 0] = -mat[0, 0]; mat[0, 1] = -mat[0, 1]; mat[1, 0] = -mat[1, 0]; mat[1, 1] = -mat[1, 1];
            return mat;
        }

        public static Matrix22 operator *(Matrix22 a, Matrix22 b)
        {
            Matrix22 ret;
            ret.m = new float[2, 2] { 
                {a[0,0] * b[0,0] + a[0,1] * b[1,0], a[0,0] * b[0,1] + a[0,1] * b[1,1] },
                {a[1,0] * b[0,0] + a[1,1] * b[1,0], a[1,0] * b[0,1] + a[1,1] * b[1,1] } };
            return ret;
        }
    }
}
