using System;
using System.Collections.Generic;
using System.Text;

namespace Math32.LinearAlgebra
{
    public struct Matrix44 //Could be optimized using unsafe
    {
        public float[,] m;

        public static Matrix44 Identity
        {
            get
            {
                Matrix44 i;
                i.m = new float[4, 4] { { 1, 0, 0, 0 }, { 0, 1, 0, 0 }, { 0, 0, 1, 0 }, { 0, 0, 0, 1 } };
                return i;
            }
        }

        public float this[int i, int j] { get { return m[i, j]; } set { m[i, j] = value; } }


        public void Transform(Matrix44 transform) { this = transform * this; }
        public void Translate(float dX, float dY, float dZ) { this = Matrix44.Translation(dX, dY, dZ) * this; }
        public static Matrix44 Translation(float dX, float dY, float dZ)
        {
            Matrix44 ret;
            ret.m = new float[4, 4] { { 1, 0, 0, 0 }, { 0, 1, 0, 0 }, { 0, 0, 1, 0 }, { dX, dY, dZ, 1 } };
            return ret;
        }

        public void ScaleX(float scaleFactor) { this = Matrix44.XScaling(scaleFactor) * this; }
        public static Matrix44 XScaling(float scaleFactor)
        {
            Matrix44 ret;
            ret.m = new float[4, 4] { { scaleFactor, 0, 0, 0 },
                                      { 0, 1, 0, 0 }, 
                                      { 0, 0, 1, 0 }, 
                                      {0, 0, 0, 1} };
            return ret;
        }
        public void ScaleY(float scaleFactor) { this = Matrix44.YScaling(scaleFactor) * this; }
        public static Matrix44 YScaling(float scaleFactor)
        {
            Matrix44 ret;
            ret.m = new float[4, 4] { { 1, 0, 0, 0 },
                                      { 0, scaleFactor, 0, 0 }, 
                                      { 0, 0, 1, 0 }, 
                                      {0, 0, 0, 1} };
            return ret;
        }
        public void ScaleZ(float scaleFactor) { this = Matrix44.ZScaling(scaleFactor) * this; }
        public static Matrix44 ZScaling(float scaleFactor)
        {
            Matrix44 ret;
            ret.m = new float[4, 4] { { 1, 0, 0, 0 },
                                      { 0, 1, 0, 0 }, 
                                      { 0, 0, scaleFactor, 0 }, 
                                      {0, 0, 0, 1} };
            return ret;
        }
        public void Scale(float scaleFactor) { this = Matrix44.Scaling(scaleFactor) * this; }
        public static Matrix44 Scaling(float scaleFactor)
        {
            Matrix44 ret;
            ret.m = new float[4, 4] { { scaleFactor, 0, 0, 0 },
                                      { 0, scaleFactor, 0, 0 }, 
                                      { 0, 0, scaleFactor, 0 }, 
                                      {0, 0, 0, 1} };
            return ret;
        }
        public void ScaleVector(Vertex v) { this = Matrix44.VectorScaling(v) * this; }
        public static Matrix44 VectorScaling(Vertex v)
        {
            Matrix44 ret;
            ret.m = new float[4, 4] { { v.x, 0, 0, 0 },
                                      { 0, v.y, 0, 0 }, 
                                      { 0, 0, v.z, 0 }, 
                                      { 0, 0, 0, 1 } };
            return ret;
        }


        //public static Matrix Rotation() { }
        public void PointAt(Vertex cameraPostion, Vertex pointAt) { this *= LookAtCamera(cameraPostion, pointAt); }
        public static Matrix44 LookAtCamera(Vertex cameraPosition, Vertex pointAt)
        {
            /*VertexStruct s, u, upV;
            Matrix ret;

            upV.x = 0f;
            upV.y = 1f;
            upV.z = 0f;
            
            s = new VertexStruct();
            upV = new VertexStruct();
            s = pointAt - cameraPostion;
            s.y = pointAt.y - cameraPostion.y;
            s.z = pointAt.z - cameraPostion.z;
            upV.x = 0f;
            upV.y = 1f;
            upV.z = 0f;

            u = VertexStruct.CrossProduct(upV, s);
            upV = VertexStruct.CrossProduct(s, u);
            VertexStruct.Normalize(s);
            VertexStruct.Normalize(upV);

            ret.m = new float[4, 4] {{u.x, u.y, u.z, 0f},
                                    {upV.x, upV.y, upV.z, 0f},
                                    {s.x, s.y, s.z, 0f},
                                    {cameraPostion.x, cameraPostion.y, cameraPostion.z, 1.0f}};
            return ret;*/

            Vertex x, y, z, upV;
            Matrix44 ret;

            upV = new Vertex();
            upV.x = 0f;
            upV.y = 1f;
            upV.z = 0f;

            z = Vertex.Normalize(pointAt - cameraPosition);
            x = Vertex.Normalize(Vertex.CrossProduct(upV, z));
            y = Vertex.CrossProduct(z, x);



            ret.m = new float[4, 4] {{x.x, y.x, z.x, 0f},
                                    {x.y, y.y, z.y, 0f},
                                    {x.z, y.z, z.z, 0f},
                                    {-Vertex.DotProduct(x,cameraPosition), 
                                        -Vertex.DotProduct(y,cameraPosition), 
                                        -Vertex.DotProduct(z,cameraPosition), 1.0f}};
            return ret;



        }

        public void Rotate(float roll, float pitch, float yaw) { this = Matrix44.Rotation(roll, pitch, yaw) * this; }
        public static Matrix44 Rotation(float roll, float pitch, float yaw)
        {
            return RotationY(yaw) * Matrix44.RotationZ(roll) * Matrix44.RotationX(pitch);
        }

        public void RotateX(float degrees) { this = Matrix44.RotationX(degrees) * this; }
        public static Matrix44 RotationX(float degrees)
        {
            Matrix44 ret;
            ret.m = new float[4, 4] {{1,0,0,0},
                                    {0,(float)Math.Cos(degrees*(Math.PI/180)), -(float)Math.Sin(degrees*(Math.PI/180)),0},
                                    {0,(float)Math.Sin(degrees*(Math.PI/180)), (float)Math.Cos(degrees*(Math.PI/180)), 0},
                                    {0,0,0,1}};
            return ret;
        }

        public void RotateY(float degrees) { this = Matrix44.RotationY(degrees) * this; }
        public static Matrix44 RotationY(float degrees)
        {
            Matrix44 ret;
            ret.m = new float[4, 4] {{(float)Math.Cos(degrees*(Math.PI/180)), 0, (float)Math.Sin(degrees*(Math.PI/180)), 0},
                                    {0, 1, 0, 0},
                                    {-(float)Math.Sin(degrees*(Math.PI/180)), 0, (float)Math.Cos(degrees*(Math.PI/180)), 0},
                                    {0, 0, 0, 1}};
            return ret;
        }

        public void RotateZ(float degrees) { this = Matrix44.RotationZ(degrees) * this; }
        public static Matrix44 RotationZ(float degrees)
        {
            Matrix44 ret;
            ret.m = new float[4, 4] {{(float)Math.Cos(degrees*(Math.PI/180)), -(float)Math.Sin(degrees*(Math.PI/180)), 0, 0},
                                    {(float)Math.Sin(degrees*(Math.PI/180)), (float)Math.Cos(degrees*(Math.PI/180)), 0, 0},
                                    {0,0,1,0},
                                    {0,0,0,1}};
            return ret;
        }

        public void RotateVector(float degrees, Vertex vector) { this = Matrix44.RotationVector(degrees, vector) * this; }
        public static Matrix44 RotationVector(float degrees, Vertex vector)
        {
            Matrix44 ret;
            float cos = (float)Math.Cos(degrees);
            float sin = (float)Math.Sin(degrees);


            ret.m = new float[4, 4] {{((vector.x*vector.x)*(1.0f-cos))+cos, 
                                     ((vector.x*vector.y)*(1.0f-cos))+(vector.z*sin),
                                     ((vector.z*vector.x)*(1.0f-cos))-(vector.y*sin),
                                     0},
                                     {((vector.x*vector.y)*(1.0f-cos))-(vector.z*sin),
                                     ((vector.y*vector.y)*(1.0f-cos))+cos,
                                     ((vector.z*vector.y)*(1.0f-cos))+(vector.x*sin),
                                     0},
                                     {((vector.x*vector.z)*(1.0f-cos))+(vector.y*sin),
                                     ((vector.y*vector.z)*(1.0f-cos))-(vector.x*sin),
                                     ((vector.z*vector.z)*(1.0f-cos))+cos,
                                     0},
                                     {0,0,0,1}};
            return ret;
        }

        public static Matrix44 TransposedMatrix(Matrix44 src)
        {
            Matrix44 ret = new Matrix44();
            ret[0, 0] = src[0, 0];
            ret[1, 0] = src[0, 1];
            ret[2, 0] = src[0, 2];
            ret[3, 0] = 0f;
            ret[0, 1] = src[1, 0];
            ret[1, 1] = src[1, 1];
            ret[2, 1] = src[1, 2];
            ret[3, 1] = 0f;
            ret[0, 2] = src[2, 0];
            ret[1, 2] = src[2, 1];
            ret[2, 2] = src[2, 2];
            ret[3, 2] = 0f;
            ret[0, 3] = (-src[0, 3] * ret[0, 0]) + (-src[1, 3] * ret[0, 1]) + (-src[2, 3] * ret[0, 2]);
            ret[1, 3] = (-src[0, 3] * ret[1, 0]) + (-src[1, 3] * ret[1, 1]) + (-src[2, 3] * ret[1, 2]);
            ret[2, 3] = (-src[0, 3] * ret[2, 0]) + (-src[1, 3] * ret[2, 1]) + (-src[2, 3] * ret[2, 2]);
            ret[3, 3] = 1.0f;
            return ret;
        }
        public void Transpose() { this = Matrix44.TransposedMatrix(this); }

        public void Invert() { this = Matrix44.Inverse(this); }
        public static Matrix44 Inverse(Matrix44 mat)
        {
            //this method uses the blockwise inversion algorithm
            //http://en.wikipedia.org/wiki/Invertible_matrix
            /*
             * [ a b ]
             * [ c d ]
            */

            Matrix22 a, invA, b, c, d, resA, resB, resC, resD, dcab;

            a.m = new float[2, 2] { { mat[0, 0], mat[0, 1] }, { mat[1, 0], mat[1, 1] } };
            invA = Matrix22.Inverse(a);
            b.m = new float[2, 2] { { mat[0, 2], mat[0, 3] }, { mat[1, 2], mat[1, 3] } };
            c.m = new float[2, 2] { { mat[2, 0], mat[2, 1] }, { mat[3, 0], mat[3, 1] } };
            d.m = new float[2, 2] { { mat[2, 2], mat[2, 3] }, { mat[3, 2], mat[3, 3] } };

            dcab = Matrix22.Inverse(d - c * invA * b);

            resA = invA + invA * b * dcab * c * invA;
            resB = -(invA * b * dcab);
            resC = -dcab * c * invA;
            resD = dcab;

            mat[0, 0] = resA[0, 0]; mat[0, 0] = resA[0, 1]; mat[0, 0] = resB[0, 0]; mat[0, 0] = resB[0, 1];
            mat[1, 0] = resA[1, 0]; mat[1, 1] = resA[1, 1]; mat[1, 2] = resB[1, 0]; mat[1, 3] = resB[1, 1];
            mat[2, 0] = resC[0, 0]; mat[2, 1] = resC[0, 1]; mat[2, 2] = resD[0, 0]; mat[2, 3] = resD[0, 1];
            mat[3, 0] = resC[1, 0]; mat[3, 1] = resC[1, 1]; mat[3, 2] = resD[1, 0]; mat[3, 3] = resD[1, 1];

            return mat;
        }

        public static Matrix44 operator *(Matrix44 a, Matrix44 b)
        {
            Matrix44 ret;
            ret.m = new float[4, 4];
            ret[0, 0] = a[0, 0] * b[0, 0] + a[0, 1] * b[1, 0] + a[0, 2] * b[2, 0] + a[0, 3] * b[3, 0];
            ret[0, 1] = a[0, 0] * b[0, 1] + a[0, 1] * b[1, 1] + a[0, 2] * b[2, 1] + a[0, 3] * b[3, 1];
            ret[0, 2] = a[0, 0] * b[0, 2] + a[0, 1] * b[1, 2] + a[0, 2] * b[2, 2] + a[0, 3] * b[3, 2];
            ret[0, 3] = a[0, 0] * b[0, 3] + a[0, 1] * b[1, 3] + a[0, 2] * b[2, 3] + a[0, 3] * b[3, 3];
            ret[1, 0] = a[1, 0] * b[0, 0] + a[1, 1] * b[1, 0] + a[1, 2] * b[2, 0] + a[1, 3] * b[3, 0];
            ret[1, 1] = a[1, 0] * b[0, 1] + a[1, 1] * b[1, 1] + a[1, 2] * b[2, 1] + a[1, 3] * b[3, 1];
            ret[1, 2] = a[1, 0] * b[0, 2] + a[1, 1] * b[1, 2] + a[1, 2] * b[2, 2] + a[1, 3] * b[3, 2];
            ret[1, 3] = a[1, 0] * b[0, 3] + a[1, 1] * b[1, 3] + a[1, 2] * b[2, 3] + a[1, 3] * b[3, 3];
            ret[2, 0] = a[2, 0] * b[0, 0] + a[2, 1] * b[1, 0] + a[2, 2] * b[2, 0] + a[2, 3] * b[3, 0];
            ret[2, 1] = a[2, 0] * b[0, 1] + a[2, 1] * b[1, 1] + a[2, 2] * b[2, 1] + a[2, 3] * b[3, 1];
            ret[2, 2] = a[2, 0] * b[0, 2] + a[2, 1] * b[1, 2] + a[2, 2] * b[2, 2] + a[2, 3] * b[3, 2];
            ret[2, 3] = a[2, 0] * b[0, 3] + a[2, 1] * b[1, 3] + a[2, 2] * b[2, 3] + a[2, 3] * b[3, 3];
            ret[3, 0] = a[3, 0] * b[0, 0] + a[3, 1] * b[1, 0] + a[3, 2] * b[2, 0] + a[3, 3] * b[3, 0];
            ret[3, 1] = a[3, 0] * b[0, 1] + a[3, 1] * b[1, 1] + a[3, 2] * b[2, 1] + a[3, 3] * b[3, 1];
            ret[3, 2] = a[3, 0] * b[0, 2] + a[3, 1] * b[1, 2] + a[3, 2] * b[2, 2] + a[3, 3] * b[3, 2];
            ret[3, 3] = a[3, 0] * b[0, 3] + a[3, 1] * b[1, 3] + a[3, 2] * b[2, 3] + a[3, 3] * b[3, 3];

            /*for (int k = 0; k < 4; k++)
            {
                for (int l = 0; l < 4; l++)
                {

                    for (int i = 0; i < 4; i++)
                    {
                        float f1 = a[k, i];
                        float f2 = b[i, l];

                        ret.m[k, l] += f1 * f2;
                    }
                }
            }*/

            return ret;
        }

        public static bool operator ==(Matrix44 a, Matrix44 b)
        {
            for (byte i = 0; i < 4; i++)
            {
                for (byte j = 0; j < 4; j++) { if (a[i, j] != b[i, j]) { return false; } }
            }
            
            return true;
        }
        public static bool operator !=(Matrix44 a, Matrix44 b) { return !(a==b); }
        public override bool Equals(object obj)
        {
            if (obj is Matrix44)
            {
                Matrix44 mat = (Matrix44)obj;
                return this == mat;
            }
            return false;
        }

       
    }
}
