using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX;


namespace Motr3D.D3DImplementation.Implementations
{
    public class D3DCamera: Object3DImplementation, ICameraImplementation
    {
        private Texture renderTarget;
        private Surface renderSurface;
        private Device device;
        private Camera camera;
        private Sprite sprite;
        private object objectLock = new object();

        public D3DCamera(Motr3D.Camera camera, Device device):base(camera)
        {
            this.device = device;
            this.camera = camera;
            AdjustDepthStencil();
            renderTarget = new Texture(device, camera.Width, camera.Height, 1, Usage.RenderTarget, Format.A8R8G8B8, Pool.Default);
            renderSurface = renderTarget.GetSurfaceLevel(0);
            sprite = new Sprite(device);
            camera.Resized += new CameraEventHandler(CameraResized);
        }

        protected override void DisposeImplementation()
        {
            renderTarget.Dispose();
            renderSurface.Dispose();
            sprite.Dispose();
        }

        public void Target()
        {
            //Concurrency should be handled within Motr3D
            lock (objectLock) { device.SetRenderTarget(0, renderSurface); }
        }

        public override void DrawSelf() 
        {
            lock (objectLock)
            {
                sprite.Begin(SpriteFlags.None);
                sprite.Draw(
                    renderTarget,
                    new Rectangle(0, 0, camera.Width, camera.Height),
                    new Vector3(0, 0, 0),
                    new Vector3(camera.Location.X, camera.Location.Y, 0),
                    Color.White);
                sprite.End();
            }
        }
        protected void AdjustDepthStencil()
        {
            int height, width;
            if (device.Viewport.Height > camera.Height) { height = device.Viewport.Height; }
            else { height = camera.Height; }
            if (device.Viewport.Width > camera.Width) { width = device.Viewport.Width; }
            else { width = camera.Width; }
            device.DepthStencilSurface =
                device.CreateDepthStencilSurface(width, height, DepthFormat.D16, MultiSampleType.None, 0, false);
        }
        private void CameraResized(Camera camera) { Update(); }
        public override void Update()
        {
            lock (objectLock)
            {
                DisposeImplementation();
                AdjustDepthStencil();
                renderTarget = new Texture(device, camera.Width, camera.Height, 1, Usage.RenderTarget, Format.A8R8G8B8, Pool.Default);
                renderSurface = renderTarget.GetSurfaceLevel(0);
                sprite = new Sprite(device);
            }
            
        }
        public override void Optimize() { }
    }
}
