using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX;

namespace Motr3D.D3DImplementation.Implementations
{
    internal class D3DMesh : D3DPrimitive
    {
        private const float ADJACENCY_TOLERANCE = 0.000001f;
        
        private Mesh mesh;
        private int[] adjacendyList;

        public D3DMesh(Motr3D.Primitives.Mesh p, Device device) : base(p, device)
        {
            
            
            //check with device caps is the device can support an indexbuffer 
            if (p.Indices.Length > ushort.MaxValue)
            {
                //short[] indices = new short[ushort.MaxValue];
                //mesh.SetIndexBufferData(p.Indices.CopyTo, LockFlags.None); 
                mesh = Mesh.Teapot(device);
            }
            else
            {
                mesh = new Mesh(p.Indices.Length, p.Vertices.Length, MeshFlags.WriteOnly, VertexFormats.Position | VertexFormats.Diffuse, device);
                mesh.SetVertexBufferData(p.Vertices, LockFlags.None);
                mesh.SetIndexBufferData(p.Indices, LockFlags.None); 
            }
            
        }

        protected override void DisposeImplementation() { mesh.Dispose(); }

        public override void DrawSelf() { mesh.DrawSubset(0); }
        public override void Update() { }
        public override void Optimize() 
        {
            adjacendyList = new int[mesh.NumberFaces];

            mesh.GenerateAdjacency(ADJACENCY_TOLERANCE, adjacendyList);

            mesh.OptimizeInPlace( MeshFlags.OptimizeVertexCache, adjacendyList);
        }
    }
}
