using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX;

namespace Motr3D.D3DImplementation.Implementations
{
    public class D3DText2D: Object3DImplementation
    {
        private Device device;
        private Sprite sprite;
        private Font font;
        private Motr3D.Primitives.Text2D text2D;

        public D3DText2D(Motr3D.Primitives.Text2D text2D, Device device): base(text2D)
        {
            this.device = device;
            this.text2D = text2D;
            sprite = new Sprite(device);
            sprite.Transform = Microsoft.DirectX.Matrix.Scaling(1, -1, 1);
            font = new Microsoft.DirectX.Direct3D.Font(device, text2D.Font);
        }

        protected override void DisposeImplementation()
        {
            sprite.Dispose();
            font.Dispose();
        }

        public override void DrawSelf()
        {
            sprite.Begin(SpriteFlags.AlphaBlend | SpriteFlags.ObjectSpace);
            font.DrawText(sprite, text2D.CurrentText, 0, 0, text2D.Color); // Very Expensive
            sprite.End();
        }

        public override void Update()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override void Optimize()
        {
            font.PreloadText(text2D.CurrentText);
            font.PreloadGlyphs(0, 255);
            font.PreloadCharacters(0, 255);
        }
    }

    internal class D3DText3D : Object3DImplementation
    {
        private Device device;
        private Mesh mesh;
        private Motr3D.Primitives.Text3D text3D;

        public D3DText3D(Motr3D.Primitives.Text3D text3D, Device device)
            : base(text3D)
        {
            this.device = device;
            this.text3D = text3D;
            mesh = Mesh.TextFromFont(device, text3D.Font, text3D.CurrentText, 10, text3D.Extrusion);

        }

        protected override void DisposeImplementation()
        {
            mesh.Dispose();
        }

        public override void Update()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override void DrawSelf()
        {
            mesh.DrawSubset(0);
        }

        public override void Optimize()
        {
            throw new Exception("The method or operation is not implemented.");
        }
    }
}
