using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX;

namespace Motr3D.D3DImplementation.Implementations
{
    internal abstract class D3DUserPrimitive: D3DPrimitive
    {
        protected VertexBuffer vertexBuffer;
        protected Motr3D.Primitives.VertexPrimitive primitive;
        
        public D3DUserPrimitive(Motr3D.Primitives.VertexPrimitive p, Device device) : base(p, device)
        {
            this.primitive = p;
            vertexBuffer = new VertexBuffer(
              typeof(Math32.LinearAlgebra.Vertex),
              p.Vertices.Length,
              device,
              Usage.WriteOnly,
              VertexFormats.Diffuse | VertexFormats.Position,
              Pool.Default);
            vertexBuffer.SetData(p.Vertices, 0, LockFlags.None);
        }

        protected override void DisposeImplementation() { vertexBuffer.Dispose(); }
        public override void Optimize() { }
        public override void Update() { }
        public override void DrawSelf()
        {
            device.SetStreamSource(0, vertexBuffer, 0);
        }
    }

    internal abstract class D3DIndexedUserPrimitive : D3DUserPrimitive
    {
        protected IndexBuffer indexBuffer;

        public D3DIndexedUserPrimitive(Motr3D.Primitives.VertexPrimitive p, Device device)
            : base(p, device)
        {
            indexBuffer = new IndexBuffer(typeof(short), p.Indices.Length, device, Usage.WriteOnly, Pool.Default);
            indexBuffer.SetData(p.Indices, 0, LockFlags.None);
        }

        protected override void DisposeImplementation()
        {
            base.DisposeImplementation();
            indexBuffer.Dispose();
        }
        public override void Optimize() { base.Optimize(); }
        public override void Update() { base.Update(); }
        public override void DrawSelf()
        {
            base.DrawSelf();
            device.Indices = indexBuffer;
        }
        
    }
    internal class D3DLineStrip : D3DIndexedUserPrimitive
    {
        public D3DLineStrip(Motr3D.Primitives.LineStrip lineStrip, Device device)
            : base(lineStrip, device) { }
        
        public override void DrawSelf()
        {
            base.DrawSelf();
            device.DrawIndexedPrimitives(
                PrimitiveType.LineStrip, 0, 0, primitive.Vertices.Length, 0, primitive.Vertices.Length - 1);
        }
    }
    internal class D3DLineList : D3DUserPrimitive
    {
        public D3DLineList(Motr3D.Primitives.LineList lineList, Device device)
            : base(lineList, device) { }
        
        public override void DrawSelf()
        {
            base.DrawSelf();
            device.DrawPrimitives(PrimitiveType.LineList, 0, primitive.Vertices.Length / 2);
        }
    }
    internal class D3DPointList : D3DUserPrimitive
    {
         public D3DPointList(Motr3D.Primitives.PointList pointList, Device device)
            : base(pointList, device) { }
        public override void DrawSelf()
        {
            base.DrawSelf();
            device.DrawPrimitives(PrimitiveType.PointList, 0, primitive.Vertices.Length / 2);
        }
    }
}
