using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX;
using System.Threading;
using Motr3D.D3DImplementation.Implementations;
using Motr3D.Demo;

namespace Motr3D.D3DImplementation
{
    public class Motr3DD3DImplementation : Motr3D.Motr3DImplementation
    {
        private Device device;
        private PresentParameters presentParameters;
        private SwapChain swapChain;

        public override Motr3D.FillMode CurrentFillMode
        {
            get 
            {
                switch (device.RenderState.FillMode)
                {
                    case Microsoft.DirectX.Direct3D.FillMode.Point: return Motr3D.FillMode.POINT; break;
                    case Microsoft.DirectX.Direct3D.FillMode.Solid: return Motr3D.FillMode.SOLID; break;
                    case Microsoft.DirectX.Direct3D.FillMode.WireFrame: return Motr3D.FillMode.WIREFRAME; break;
                    default: return Motr3D.FillMode.CURRENT; break;
                }
            }
            set 
            { 
                //device.RenderState.FillMode = (Microsoft.DirectX.Direct3D.FillMode)value;
                switch (value)
                {
                    case Motr3D.FillMode.POINT: device.RenderState.FillMode = Microsoft.DirectX.Direct3D.FillMode.Point; break;
                    case Motr3D.FillMode.SOLID: device.RenderState.FillMode = Microsoft.DirectX.Direct3D.FillMode.Solid; break;
                    case Motr3D.FillMode.WIREFRAME: device.RenderState.FillMode = Microsoft.DirectX.Direct3D.FillMode.WireFrame; break;
                }
            }
        }
        public override Motr3D.CullMode CurrentCullMode 
        {
            get 
            {
                switch (device.RenderState.CullMode)
                {
                    case Microsoft.DirectX.Direct3D.Cull.Clockwise: return Motr3D.CullMode.CLOCKWISE;
                    case Microsoft.DirectX.Direct3D.Cull.CounterClockwise: return Motr3D.CullMode.COUNTERCLOCKWISE;
                    case Microsoft.DirectX.Direct3D.Cull.None: return Motr3D.CullMode.NONE;
                    default: return Motr3D.CullMode.CURRENT;
                }
            }
            set 
            {
                switch (value)
                {
                    case Motr3D.CullMode.CLOCKWISE: device.RenderState.CullMode = Microsoft.DirectX.Direct3D.Cull.Clockwise; break;
                    case Motr3D.CullMode.COUNTERCLOCKWISE: device.RenderState.CullMode = Microsoft.DirectX.Direct3D.Cull.CounterClockwise; break;
                    case Motr3D.CullMode.NONE: device.RenderState.CullMode = Microsoft.DirectX.Direct3D.Cull.None; break;
                }
            }
        }
        public override Motr3D.Lighting EnableLighting 
        { 
            get 
            {
                if (device.RenderState.Lighting == true) { return Motr3D.Lighting.YES; }
                return Motr3D.Lighting.NO;
            } 
            set 
            {
                if (value == Motr3D.Lighting.YES) { device.RenderState.Lighting = true; }
                else { device.RenderState.Lighting = false; }
            } 
        }
        public override bool DepthBuffer
        {
            get { return device.RenderState.ZBufferEnable; }
            set { device.RenderState.ZBufferEnable = value; }
        }
        private delegate void DeviceCreationDelegate(
            int adapter,
            DeviceType deviceType,
            Control control,
            CreateFlags createFlags,
            params PresentParameters[] presentParameters);
        private void CreateDevice(
            int adapter,
            DeviceType deviceType,
            Control control,
            CreateFlags createFlags,
            params PresentParameters[] presentParameters) 
        {
            device = new Device(adapter, deviceType, control, createFlags, presentParameters);
        }
        public override void InitializeImplementation(Control viewport)
        {
            this.viewport = viewport;
            Caps caps = Manager.GetDeviceCaps(Manager.Adapters.Default.Adapter, DeviceType.Hardware);
            CreateFlags flags;

            if (device != null) { device.Dispose(); }
            if (renderingControl != null) { renderingControl.Dispose(); }
            renderingControl = new Control();
            //renderingControl.CreateControl();

            if (caps.DeviceCaps.SupportsHardwareTransformAndLight) { flags = CreateFlags.HardwareVertexProcessing; }
            else { flags = CreateFlags.SoftwareVertexProcessing; }

            presentParameters = new PresentParameters();
            presentParameters.BackBufferFormat = Format.A8R8G8B8;
            presentParameters.SwapEffect = SwapEffect.Discard;
            presentParameters.Windowed = true;
            presentParameters.EnableAutoDepthStencil = true;
            presentParameters.AutoDepthStencilFormat = DepthFormat.D16;
            presentParameters.PresentFlag = PresentFlag.LockableBackBuffer;
            presentParameters.PresentationInterval = PresentInterval.Immediate;
            presentParameters.BackBufferHeight = 50;//viewport.Height;
            presentParameters.BackBufferWidth = 50;//viewport.Width;
            //Needs to be changed to accomodate software rendering
           /* if (!viewport.InvokeRequired)
            {
                device = new Device(0, DeviceType.Hardware, viewport, flags, presentParameters);
            }
            else 
            {
                viewport.Invoke(new DeviceCreationDelegate(CreateDevice), 0, DeviceType.Hardware, viewport, flags, new PresentParameters[1] { presentParameters }); 
            }*/
            /*Creating the device on viewport message loop does not work since Direct3D will set the control's 
             thread to the thread it was created on, thus the control will not receive any message from its parent*/
            device = new Device(0, DeviceType.Hardware, renderingControl, flags, presentParameters);
            device.VertexFormat = VertexFormats.Position | VertexFormats.Diffuse;
            device.DeviceReset += new System.EventHandler(DeviceReseted);
            device.DeviceLost += new EventHandler(DeviceLost);
            //swapChain = new SwapChain(device, presentParameters);         

            device.RenderState.SourceBlend = Blend.SourceAlpha;
            device.RenderState.DestinationBlend = Blend.InvSourceAlpha;
            device.RenderState.AlphaBlendEnable = true;
            device.RenderState.ZBufferEnable = true;
            ResetRenderingEngine();
        }
        
        private void DeviceReseted(object sender, EventArgs args) { ResetRenderingEngine(); }
        private void DeviceLost(object sender, EventArgs args)
        {
            InitializeImplementation(viewport); 
        }

        public override void ResetRenderingEngine()
        {
            if (swapChain != null) { swapChain.Dispose(); }
            presentParameters.BackBufferHeight = viewport.Height;
            presentParameters.BackBufferWidth = viewport.Width;
            swapChain = new SwapChain(device, presentParameters);
            foreach (KeyValuePair<Object3D, IObject3DImplementation> c in implementations)
            {
                if (c.Value is ICameraImplementation) { c.Value.Update(); }
            }
        }    

        protected override void DisposeRenderingEngine()
        {
            device.DeviceLost -= new EventHandler(DeviceLost);
            swapChain.Dispose();
            //device.Dispose();
        }

        public override void ClearTarget(Color color)
        {
            device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, color, 1.0f, 0);    
        }
        public override void BeginRendering() { device.BeginScene(); }
        public override void EndRendering() { device.EndScene(); }
        public override void DisplayRender() 
        {
            if(viewport == null){ return; }
            if (viewport.InvokeRequired) { viewport.Invoke(new MethodInvoker(DisplayRender)); }
            else { swapChain.Present(viewport); }//device.Present(viewport); }
            
        }

        public override void Target(Camera camera)
        {
            if (camera != null)
            {
                base.Target(camera);
            }
            else
            {
                //device.SetRenderTarget(0, device.GetBackBuffer(0, 0, BackBufferType.Mono));
                device.SetRenderTarget(0, swapChain.GetBackBuffer(0, BackBufferType.Mono));
            }
        }

        protected override IObject3DImplementation MakeImplementation(Object3D object3D)
        {
            //if (object3D is Motr3D.Primitives.Sphere) { return new D3DMesh((Motr3D.Primitives.Mesh)object3D, device); }
            if (object3D is Motr3D.Primitives.LineList) { return new D3DLineList((Motr3D.Primitives.LineList)object3D, device); }
            if (object3D is Motr3D.Primitives.LineStrip) { return new D3DLineStrip((Motr3D.Primitives.LineStrip)object3D, device); }
            if (object3D is Motr3D.Primitives.Text2D) { return new D3DText2D((Motr3D.Primitives.Text2D)object3D, device); }
            if (object3D is Motr3D.Primitives.PointList) 
            { 
                return new D3DPointList((Motr3D.Primitives.PointList)object3D, device); 
            }
            if (object3D is Motr3D.Primitives.Text3D) { return new D3DText3D((Motr3D.Primitives.Text3D)object3D, device); }
            if (object3D is Motr3D.Primitives.Mesh) { return new D3DMesh((Motr3D.Primitives.Mesh)object3D, device); }
            if (object3D is Motr3D.Camera) { return new D3DCamera((Camera)object3D, device); }
            
            throw new ArgumentException("A Direct3D implementation cannot be found for " + object3D.ToString());  
        }

        public override Bitmap GetLastRender()
        {
            Bitmap bmp;
            Surface surf = device.GetBackBuffer(0, 0, BackBufferType.Mono);
            Graphics g = surf.GetGraphics();
            bmp = new Bitmap(renderWidth, renderHeight, g); 
            surf.ReleaseGraphics();
            return bmp;            
        }
       /* protected struct IInfoMeshPair { public IntersectInformation ii; public Motr3D.Primitives.Primitive prim;}
        public override List<Motr3D.Primitives.Primitive> PickPrimitives(MouseEventArgs args)
        {
            Vector3 rayOrigin, rayDir, worldPos1, worldPos2;
            Microsoft.DirectX.Matrix invView;
            List<IInfoMeshPair> picked = new List<IInfoMeshPair>();
            List<Motr3D.Primitives.Primitive> pickedPrimitives = new List<Motr3D.Primitives.Primitive>();

            worldPos1.X = (((2.0f * (float)args.X) / (float)presentParameters.BackBufferWidth) - 1.0f) / device.Transform.Projection.M11;
            worldPos1.Y = -(((2.0f * (float)args.Y) / (float)presentParameters.BackBufferHeight) - 1.0f) / device.Transform.Projection.M22;
            worldPos1.Z = 1.0f;
            worldPos2.X = ((float)args.X / ((float)presentParameters.BackBufferWidth / 2.0f) - 1.0f) / ( (float)viewport.Width /(float)viewport.Height );
            worldPos2.Y = 1.0f - ((float)args.Y /((float)presentParameters.BackBufferHeight /2.0f));
            worldPos2.Z = 0.0f;
            //worldPos2.X *= (float)Math.Tan(Math.PI / 16f);
            //worldPos2.Y *= (float)Math.Tan(Math.PI / 16f);

            invView = Microsoft.DirectX.Matrix.Invert(device.Transform.View);

            rayDir.X = worldPos1.X * invView.M11 + worldPos1.Y * invView.M21 + worldPos1.Z * invView.M31;
            rayDir.Y = worldPos1.X * invView.M12 + worldPos1.Y * invView.M22 + worldPos1.Z * invView.M32;
            rayDir.Z = worldPos1.X * invView.M13 + worldPos1.Y * invView.M23 + worldPos1.Z * invView.M33;
            rayOrigin.X = invView.M41;
            rayOrigin.Y = invView.M42;
            rayOrigin.Z = invView.M43;

            foreach (DictionaryEntry de in primitiveImplementations)
            {
                Mesh m;
                Vector3 objectSpaceOriginRay = rayOrigin, objectSpaceDirRay = rayDir;
                Matrix objectSpaceMat;
                IntersectInformation ii;
                Motr3D.Primitives.Primitive p;

                if (de.Key is Motr3D.Primitives.Primitive && de.Value is Mesh)
                {
                    p = de.Key as Motr3D.Primitives.Primitive;
                    m = de.Value as Mesh;
                }
                else { continue; }
                if (!p.Pickable) { continue; }
                //objectSpaceMat = Matrix.Invert(ConvertMatrix(p.CurrentWorldTransform));
                
                //objectSpaceOriginRay.TransformCoordinate(objectSpaceMat);
                //objectSpaceDirRay.TransformCoordinate(objectSpaceMat);
                if (m.Intersect(objectSpaceOriginRay, objectSpaceDirRay, out ii))
                {
                    IInfoMeshPair iimp;

                    int index = 0;
                    while (index < picked.Count) { if (picked[index++].ii.Dist < ii.Dist) { break; } }
                    iimp.ii = ii;
                    iimp.prim = p;
                    picked.Insert(index, iimp);
                }
            }

            foreach (IInfoMeshPair iimp in picked) { pickedPrimitives.Add(iimp.prim); }
            return pickedPrimitives;            
        }*/

        protected virtual Microsoft.DirectX.Matrix ConvertMatrix(Math32.LinearAlgebra.Matrix44 transform)
        {
            Microsoft.DirectX.Matrix m = new Microsoft.DirectX.Matrix();

            m.M11 = transform[0,0]; m.M12 = transform[0,1]; m.M13 = transform[0,2]; m.M14 = transform[0,3];
            m.M21 = transform[1,0]; m.M22 = transform[1,1]; m.M23 = transform[1,2]; m.M24 = transform[1,3];
            m.M31 = transform[2,0]; m.M32 = transform[2,1]; m.M33 = transform[2,2]; m.M34 = transform[2,3];
            m.M41 = transform[3,0]; m.M42 = transform[3,1]; m.M43 = transform[3,2]; m.M44 = transform[3,3];

            return m;
        }
        public override void SetPerspectiveProjection(float fieldOfView, float aspectRatio, float nearClipDistance, float farClipDistance)
        {
            device.Transform.Projection = Microsoft.DirectX.Matrix.PerspectiveFovLH(
                (float)(fieldOfView * (Math.PI / 180)), 
                aspectRatio, 
                nearClipDistance, 
                farClipDistance);
        }
        public override void SetOrthogonalProjection(float width, float height, float nearClipDistance, float farClipDistance)
        {
            device.Transform.Projection = Microsoft.DirectX.Matrix.OrthoLH(width, height, nearClipDistance, farClipDistance);
        }

        public override void TransformWorld(Math32.LinearAlgebra.Matrix44 transform) { device.Transform.World = ConvertMatrix(transform); }

        public override void TransformCamera(Math32.LinearAlgebra.Matrix44 transform)
        {
            device.Transform.View = ConvertMatrix(transform);
        }
        static void Main()
        {
            using (Motr3DDemo frm = new Motr3DDemo( new Motr3DD3DImplementation()))
            {
                frm.Show();
                Application.Run(frm);
            }
        }
    }
}
