using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;
using Motr3D.Primitives;
using Math32.LinearAlgebra;

namespace Motr3D
{
    public enum RenderState { STARTED, STOPPED }
    
    public delegate void CameraEventHandler(Camera sender);

    public class Camera: Object3D
    {
        protected List<Object3D> renderNodes = new List<Object3D>();
        protected bool enableRendering = false;
        private DockStyle dock = DockStyle.Fill;
        protected Color backgroundColor = Color.Black;
        private Motr3DViewport viewport;
        private Point location = new Point(0,0);

        protected float fieldOfView = 45.0f;
        protected float aspectRatio = 1.0f;
        protected float nearClipDistance = 0.01f;
        protected float farClipDistance = 100.0f;
        protected int width = 150, height = 150;
        protected volatile MouseEventArgs lastMouseEventArgs;

        public event CameraEventHandler RenderNodesChanged;
        public event CameraEventHandler Resized;


        public event EventHandler KeyUp;
        public event KeyEventHandler KeyDown;
        public event EventHandler KeyPress;
        public event MouseEventHandler Click;
        public event MouseEventHandler MouseUp;
        public event MouseEventHandler MouseDown;
        public event EventHandler MouseEnter;
        public event EventHandler MouseLeave;
        public event EventHandler MouseHover;
        public event MouseEventHandler MouseMove;

        public Camera(Object3D worldRoot)
        {
            renderNodes.Add(worldRoot);
            //AdaptViewPortToContainer = true;
            /*engine.ViewPort.KeyDown += new KeyEventHandler(OnKeyDown);
            engine.ViewPort.KeyPress += new KeyPressEventHandler(OnKeyPress);
            engine.ViewPort.KeyUp += new KeyEventHandler(OnKeyUp);
            engine.ViewPort.Click += new EventHandler(OnClick);
            engine.ViewPort.MouseUp += new MouseEventHandler(OnMouseUp);
            engine.ViewPort.MouseDown += new MouseEventHandler(OnMouseDown);
            engine.ViewPort.MouseEnter += new EventHandler(OnMouseEnter);
            engine.ViewPort.MouseLeave += new EventHandler(OnMouseLeave);
            engine.ViewPort.MouseHover += new EventHandler(OnMouseHover);
            engine.ViewPort.MouseMove += new MouseEventHandler(OnMouseMove);*/
        }
        
        public virtual Color BackgroundColor { get { return backgroundColor; } set { backgroundColor = value; } }
        public virtual List<Object3D> RenderNodes 
        { 
            get { return renderNodes; }
            set { lock (renderNodes) { renderNodes = value; } OnRenderNodesChanged(); } 
        }
        public virtual float FieldOfView { get { return fieldOfView; } set { fieldOfView = value; } }
        public virtual float AspectRatio { get { return aspectRatio; } set { aspectRatio = value; } }
        public virtual float NearClipDistance { get { return nearClipDistance; } set { nearClipDistance = value; } }
        public virtual float FarClipDistance{ get { return farClipDistance; } set { farClipDistance = value; } }
        public virtual int Width { get { return width; } set { width = value; } }
        public virtual int Height { get { return height; } set { height = value; } }
        public virtual Point Location { get { return location; } set { location = value; } }
        public virtual DockStyle Dock { get { return dock; } set { dock = value; } }
        public virtual Motr3DViewport Viewport 
        { 
            get { return viewport; } 
            protected internal set
            {
                if (viewport != null) { viewport.Resize -= new EventHandler(ViewportResized); }
                viewport = value;
                viewport.Resize += new EventHandler(ViewportResized);
                ViewportResized(null, null);
            } 
        }

        protected void ViewportResized(object sender, EventArgs args)
        {
            switch (dock)
            {
                case DockStyle.None: return;
                case DockStyle.Fill: height = viewport.Height; width = viewport.Width; goto default;
                default: OnResized(); break;
            }
            aspectRatio = (float)width / (float)height; 
        }

        protected virtual void RegisterMouseEventArgs(object sender, MouseEventArgs args) {  }

        protected override void DrawSelf(Camera camera)
        {
        
        }
       /* protected virtual void DrawSelf()
        {
            Vertex t = new Vertex(1, 0, 0);
            Vertex pos = new Vertex(1, 0, 0);
            pos.Transform(currentTranslationMatrix);

            t.Transform(currentScalingMatrix *
                currentRotationMatrix *
                currentTranslationMatrix);
            engine.TransformCamera(Matrix44.LookAtCamera(pos, t));
        }*/

        public void OnRenderNodesChanged() { if (RenderNodesChanged != null) { RenderNodesChanged(this); } }

        public void OnResized() { if (Resized != null) { Resized(this); } }

        protected void OnClick(object sender, EventArgs args) { OnClick(lastMouseEventArgs); }
        public void OnClick(MouseEventArgs args) { if (Click != null) { Click(this, args); } }
        protected void OnMouseMove(object sender, MouseEventArgs args) { OnMouseMove(args); }
        public void OnMouseMove(MouseEventArgs args)
        {
            //lastMouseEventArgs = args; if (picking) { doPicking = true; }
            if (MouseMove != null) { MouseMove(this, args); } 
        }
        protected void OnMouseUp(object sender, MouseEventArgs args) { OnMouseUp(args); }
        public void OnMouseUp(MouseEventArgs args) { if (MouseUp != null) { MouseUp(this, args); } }
        protected void OnMouseDown(object sender, MouseEventArgs args) { OnMouseDown(args); }
        public void OnMouseDown(MouseEventArgs args) { if (MouseDown != null) { MouseDown(this, args); } }
        //protected void OnMouseEnter(object sender, EventArgs args) { OnMouseEnter(); }
        //public void OnMouseEnter() { engine.ViewPort.Focus(); if (MouseEnter != null) { MouseEnter(this, null); } }
        protected void OnMouseLeave(object sender, EventArgs args) { OnMouseLeave(); }
        public void OnMouseLeave() { if (MouseLeave != null) { MouseLeave(this, null); } }
        //protected void OnMouseHover(object sender, EventArgs args) { OnMouseHover(); if (picking) { doPicking = true; } }
        public void OnMouseHover() { if (MouseHover != null) { MouseHover(this, null); } }

        protected void OnKeyDown(object sender, KeyEventArgs args) { OnKeyDown(args); }
        public void OnKeyDown(KeyEventArgs args) { if (KeyDown != null) { KeyDown(this, args); } }
        protected void OnKeyPress(object sender, KeyPressEventArgs args) { OnKeyPress(args); }
        public void OnKeyPress(KeyPressEventArgs args) { if (KeyPress != null) { KeyPress(this, args); } }
        protected void OnKeyUp(object sender, KeyEventArgs args) { OnKeyUp(args); }
        public void OnKeyUp(KeyEventArgs args) { if (KeyUp != null) { KeyUp(this, args); } }
    }
}
