using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Motr3D.Prebuilt;
using Motr3D.Primitives;
using System.Threading;
using Math32.LinearAlgebra;

namespace Motr3D.Demo
{
    public partial class Motr3DDemo : Form
    {
        private const int DEFAULT_NUMBER_OF_STARS = 5000;
        
        protected Form earthView = new Form();
        protected Reference universe = new Reference(), solarSystem = new Reference();
        //No pluto since its no longer a planet...
        private Planet sun, mercury, venus, earth, moon, mars, jupiter, saturn, uranus, neptune; 
        protected System.Threading.Timer timer;
        protected Camera freeCamera, topCamera, moonCamera; 
        protected int lastMouseX;
        protected int lastMouseY;
        private PointList stars;

        private Motr3DViewport freeViewport;

        private Motr3DDemo() { InitializeComponent(); }
        public Motr3DDemo(IMotr3DImplementation implementation )
        {
            InitializeComponent();
            this.ClientSize = new System.Drawing.Size(640, 480);
            this.Text = "Motr Demo";
            InitializeUniverse();
            
            lastMouseX = this.Width/2;
            lastMouseY = this.Height/2;

            freeViewport = new Motr3DViewport(implementation);
            //earthViewport = new Motr3DViewport(implementation);

            freeViewport.KeyDown += new KeyEventHandler(OnKeyDown);
            freeViewport.MouseMove += new MouseEventHandler(OnMouseMove);

            this.Controls.Add(freeViewport);
            freeViewport.AddCameras(freeCamera, topCamera, moonCamera);
            freeViewport.Dock = DockStyle.Fill;

            //earthView.Controls.Add(earthViewport);
            //earthViewport.Dock = DockStyle.Fill;
            //earthViewport.AddCameras(camera2);
            
            //earthView.Show();
            //camera1.StartRendering();
            //camera2.StartRendering();
        }

        protected void OnKeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.A) { freeCamera.DirectionnalTranslate(0, 0, 1); }
            else if (e.KeyCode == Keys.D) { freeCamera.DirectionnalTranslate(0, 0, -1); }
            else if (e.KeyCode == Keys.W) { freeCamera.DirectionnalTranslate(1, 0, 0); }
            else if (e.KeyCode == Keys.S) { freeCamera.DirectionnalTranslate(-1, 0, 0); }
            else if (e.KeyCode == Keys.Add) { solarSystem.Scale(1.1f); }
            else if (e.KeyCode == Keys.Subtract) { solarSystem.Scale(0.9f); }
        }

        protected void OnMouseMove(object sender,MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                freeCamera.RotateZ((e.Y - lastMouseY)/2);
                freeCamera.RotateY(-(e.X - lastMouseX)/2);
            }
          
            lastMouseX = e.X;
            lastMouseY = e.Y;
            
        }

        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            base.OnClosing(e);
            freeViewport.Dispose();
        }

        protected virtual void InitializeUniverse()
        {                        
            sun = new Planet("Sun", Color.Yellow, 15.0f, 0, 0, 0, 0);
            mercury = new Planet("Mercury", Color.DarkGray, 0.5f, 0, 2.0f, 30, 2);

            venus = new Planet("Venus", Color.Beige, 1.5f, -20, 1.5f, 50, 1.5f);

            earth = new Planet("Earth", Color.Blue, 1.7f, 15, 1, 70, -1.2f);
            moon = new Planet("Moon", Color.White, 0.4f, 0, 0, 4, 0.5f);
            earth.AddChildToPlanet(moon);

            mars = new Planet("Mars", Color.Brown, 1, -4, -0.8f, 90, -1.0f);
            jupiter = new Planet("Jupiter", Color.SandyBrown, 7, -40, 0.5f, 120, 0.6f);
            saturn = new Planet("Saturn", Color.SeaShell, 5, 90, 0.4f, 150, -0.4f);
            uranus = new Planet("Uranus", Color.Turquoise, 4, 30, -0.7f, 200, 0.35f);
            neptune = new Planet("Neptune", Color.Azure, 4, -10, 0.2f, 215, -0.30f);
            /*mars = new CelestialObject(0.8f, Color.Red, "Mars", universe); solarSystem.Add(mars);
            mars.orbit = 0.9f; mars.rotation = -0.7f; mars.celestialObject.Translate(nD + 13, -0.3f, 0);

            jupiter = new CelestialObject(10.0f, Color.RosyBrown, "Jupiter", universe); solarSystem.Add(jupiter);
            jupiter.orbit = 0.4f; jupiter.rotation = -0.3f; jupiter.celestialObject.Translate(nD + 30, -0.3f, 0);

            saturn = new CelestialObject(6.0f, Color.Ivory, "Saturn", universe); solarSystem.Add(saturn);
            saturn.orbit = -0.2f; saturn.rotation = -0.4f; saturn.celestialObject.Translate(nD + 43, 0.2f, 0); saturn.celestialObject.RotateZ(-20);

            uranus = new CelestialObject(3.1f, Color.Turquoise, "Uranus", universe); solarSystem.Add(uranus);
            uranus.orbit = 0.1f; uranus.rotation = -0.35f; uranus.celestialObject.Translate(nD + 57, 0, 0);

            neptune = new CelestialObject(3.0f, Color.Indigo, "Neptune", universe); solarSystem.Add(neptune);
            neptune.orbit = 0.07f; neptune.rotation = 0.30f; neptune.celestialObject.Translate(nD + 70, 1, 0);*/

            solarSystem.AddChildren(sun, mercury, venus, earth, mars, jupiter, saturn, uranus, neptune);

            freeCamera = new Camera(universe);
            freeCamera.FarClipDistance = 10000.0f;
            freeCamera.Translate(-100, 0.0001f, 0.0001f);

            topCamera = new Camera(universe);
            topCamera.Height = 200; topCamera.Width = 200;
            topCamera.FarClipDistance = 10000.0f;
            topCamera.Dock = DockStyle.None;
            topCamera.Translate(0, 500, 0);
            topCamera.RotateZ(90);

            moonCamera = new Camera(universe);
            moonCamera.Height = 200; moonCamera.Width = 200;
            moonCamera.FarClipDistance = 10000.0f; moonCamera.NearClipDistance = 10.0f;
            moonCamera.Dock = DockStyle.None;
            moonCamera.Location = new Point(0, topCamera.Height);

            moon.AddChildToPlanet(moonCamera);
            universe.AddChildren(freeCamera, topCamera, solarSystem);
            GenerateStars(DEFAULT_NUMBER_OF_STARS);
        }

        private void GenerateStars(int numberOfStars)
        {
            const double limit = 500.0, span = 10000.0;
            
            if (stars != null) { stars.Dispose(); }
            stars = new PointList();
            Random random = new Random();
            Vertex[] vertices = new Vertex[numberOfStars];

            for (int i = 0; i < numberOfStars; i++)
            {
                Vertex vert;
                
                double x = random.NextDouble() * span;
                x -= span / 2;
                double y = random.NextDouble() * span;
                y -= span / 2;
                double z = random.NextDouble() * span;
                z -= span / 2;

                vert = new Vertex((float)x, (float)y, (float)z, Color.White);
                if (vert.Magnitude() < limit) { i--; continue; }
                vertices[i] = vert;
            }

            stars.Vertices = vertices;
            universe.AddChild(stars);
        }

        private void settingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            settingsGroupBox.Visible = !settingsGroupBox.Visible;
        }

        private void refAxesCheckBox_Click(object sender, EventArgs e)
        {
            foreach (Object3D p in solarSystem.Children) 
            {
                if (!(p is Planet)) { continue; }
                p.ShowAxes = refAxesCheckBox.Checked; 
            }
            moon.ShowAxes = refAxesCheckBox.Checked;
        }

        private void wireframeRadioButton_Click(object sender, EventArgs e)
        {
            universe.FillMode = FillMode.WIREFRAME;
            solidRadioButton.Checked = false;
            pointsRadioButton.Checked = false;
        }

        private void solidRadioButton_Click(object sender, EventArgs e)
        {
            universe.FillMode = FillMode.SOLID;
            wireframeRadioButton.Checked = false;
            pointsRadioButton.Checked = false;
        }

        private void pointsRadioButton_Click(object sender, EventArgs e)
        {
            universe.FillMode = FillMode.POINT;
            wireframeRadioButton.Checked = false;
            solidRadioButton.Checked = false;
        }

        private void backgroundColorButton_Click(object sender, EventArgs e)
        {
            colorDialog.ShowDialog();
            freeCamera.BackgroundColor = colorDialog.Color;
            topCamera.BackgroundColor = colorDialog.Color;
            moonCamera.BackgroundColor = colorDialog.Color;
            backgroundColorPanel.BackColor = colorDialog.Color;
        }
    }
}