using System;
using System.Collections.Generic;
using System.Text;
using Motr3D.Prebuilt;
using Motr3D.Primitives;
using System.Drawing;
using System.Threading;

namespace Motr3D.Demo
{
    internal class Planet: Reference
    {
        private const int ROTATION_INTERVAL_MS = 25;

        private Timer timer;
        private long lastRotation = DateTime.Now.Ticks;
        private Text2D label;
        private Reference orbit = new Reference();
        private Sphere planet;
        private float rotation, period;
        
        public Planet(
            string name, 
            Color color, 
            float radius, 
            float inclination, 
            float rotation, 
            float distance, 
            float period)
        {
            planet = new Sphere(radius, 20, 20);
            planet.Color = color;
            timer = new Timer(new TimerCallback(DoRotation));
            this.rotation = rotation;
            timer.Change(0, ROTATION_INTERVAL_MS);
            label = new Text2D(name, "Arial", 8.0f); label.Scale(0.2f);
            label.Color = color;
            label.Translate(0, radius + 6f, 0);
            planet.RotateZ(inclination);
            this.period = period;
            orbit.Translate(0, 0, distance);
            orbit.AddChildren(planet, label);
            AddChildren(orbit);
        }

        public Visibility ShowLabel
        {
            get { return label.Visibility; }
            set { label.Visibility = value; }
        }
        public override bool ShowAxes
        {
            get { return planet.ShowAxes; }
            set { planet.ShowAxes = value; }
        }
        public virtual void AddChildToPlanet(Object3D child)
        {
            orbit.AddChild(child);
        }
        
        private void DoRotation(Object state)
        {
            planet.RotateY(rotation);
            //RotateY((DateTime.Now.Ticks - lastRotation) * period / 10000);
            RotateY(period/2);

            lastRotation = DateTime.Now.Ticks;
        }
    }
}
