using System;
using System.Collections.Generic;
using System.Text;
using Math32.LinearAlgebra;
using Motr3D.Primitives;

namespace Motr3D.Prebuilt
{
    public class Sphere : Mesh
    {
        protected float radius;
        protected int nRings;
        protected int nSegments;    

        public Sphere(float radius, int rings, int segments):base()
        {
            this.radius = radius;
            this.nRings = rings;
            this.nSegments = segments;
            GenerateVertexData();
        }

        protected override void GenerateVertexData()
        {
            //http://www.ogre3d.org/wiki/index.php/ManualSphereMeshes

            int verticeIndex = 0;
            int indiceIndex = 0;
            float dRingAngle = (float)(Math.PI / nRings);
            float dSegmentAngle = (float)(2 * Math.PI / nSegments);
            
            Vertex[] vertices = new Vertex[(nRings+1)*(nSegments+1)];
            short[] indices = new short[6 * nRings * (nSegments+1)];

            for (int ring = 0; ring <= nRings; ring++)
            {
                float r0 = (float)(radius * Math.Sin(ring * dRingAngle));
                float y0 = (float)(radius * Math.Cos(ring * dRingAngle));

                for (int seg = 0; seg <= nSegments; seg++)
                {
                    float x0 = (float)(r0 * Math.Sin(seg * dSegmentAngle));
                    float z0 = (float)(r0 * Math.Cos(seg * dSegmentAngle));

                    vertices[verticeIndex].x = x0;
                    vertices[verticeIndex].y = y0;
                    vertices[verticeIndex].z = z0;
                    vertices[verticeIndex].color = Color.ToArgb();

                    if (ring < nRings)
                    {
                        indices[indiceIndex++] = (short)(verticeIndex + nSegments + 1);
                        indices[indiceIndex++] = (short)(verticeIndex);
                        indices[indiceIndex++] = (short)(verticeIndex + nSegments);
                        indices[indiceIndex++] = (short)(verticeIndex + nSegments + 1);
                        indices[indiceIndex++] = (short)(verticeIndex + 1);
                        indices[indiceIndex++] = (short)(verticeIndex);   
                    }
                    verticeIndex++;
                }
            }

            Vertices = vertices;
            Indices = indices;
        }

    }
}
