using System;
using System.Collections.Generic;
using System.Text;
using Math32.LinearAlgebra;

namespace Motr3D.Primitives
{
    public class LineStrip : VertexPrimitive
    {
        protected float[] x, y, z;
        protected float lineWidth = 1.0f;

        public LineStrip(float length, int segments)
        {
            float segLength = length / segments;
            /*vertices = new VertexStruct[segments + 1];
            indices = new short[2 * segments];*/
            x = new float[segments+1]; y = new float[segments+1]; z = new float[segments+1];

            for (int i = 0; i <= segments; i++)
            {
                x[i] = i * segLength;
                y[i] = 0;
                z[i] = 0;
            }

            GenerateVertexData();
        }
        public LineStrip(float[] x, float[] y, float[] z)
        {
            this.x = x; this.y = y; this.z = z;
            GenerateVertexData();
        }

        public float[] X { get { return x; } set { x = value; GenerateVertexData(); } }
        public float[] Y { get { return y; } set { y = value; GenerateVertexData(); } }
        public float[] Z { get { return z; } set { z = value; GenerateVertexData(); } }
        public float LineWidth { get { return lineWidth; } set { lineWidth = value; } }

        public virtual void GenerateLineStrip(float[] x, float[] y, float[] z)
        {
            this.x = x; this.y = y; this.z = z;
            GenerateVertexData();
        }

        protected override void DrawSelf(Camera camera)
        {
            //Matrix m = parent.GetAncestorsTransformMatrix();
            //viewPort.TransformWorld(currentTransform*m * Matrix.Translation(position.x, position.y, position.z));
            /*camera.RenderingEngine.TransformWorld(
                currentTransform *
                Matrix.Translation(RelativePosition.x, RelativePosition.y, RelativePosition.z)*
                m/* *
                Matrix.Translation(
                AbsolutePosition.x + RelativePosition.x,
                AbsolutePosition.y + RelativePosition.y,
                AbsolutePosition.z + RelativePosition.z)*/
                //);
            //camera.RenderingEngine.DrawPrimitive(this);
            camera.Viewport.Implementation.DrawObject(this);
        }

        protected override void GenerateVertexData()
        {
            Vertex[] vertices = new Vertex[x.Length];
            short[] indices = new short[2 * (x.Length - 1)];

            vertices[0].x = x[0];
            vertices[0].y = y[0];
            vertices[0].z = z[0];
            vertices[0].color = Color.ToArgb();
            indices[0] = 0;

            for (int i = 1; i < x.Length - 1; i++)
            {
                vertices[i].x = x[i];
                vertices[i].y = y[i];
                vertices[i].z = z[i];
                vertices[i].color = Color.ToArgb();
                indices[2 * i - 1] = (short)i;
                indices[2 * i] = (short)i;
            }

            vertices[x.Length - 1].x = x[x.Length - 1];
            vertices[x.Length - 1].y = y[x.Length - 1];
            vertices[x.Length - 1].z = z[x.Length - 1];
            vertices[x.Length-1].color = Color.ToArgb();
            indices[2 * (x.Length - 1)-1] = (short)(x.Length-1);

            Indices = indices;
            Vertices = vertices;
        }

        /*protected override void GenerateVertexData()
        {
            float segLength = length / segments;
            vertices = new VertexStruct[segments+1];
            indices = new short[2*segments];

            vertices[0].x = 0;
            vertices[0].y = 0;
            vertices[0].z = 0;
            vertices[0].color = color.ToArgb();
            indices[0] = 0;

            for (int i = 1; i < (segments); i++)
            {
                vertices[i].x = i * segLength;
                vertices[i].y = 0;
                vertices[i].z = 0;
                vertices[i].color = color.ToArgb();
                indices[2 * i - 1] = (short)i;
                indices[2 * i] = (short)i;
            }

            vertices[segments].x = length;
            vertices[segments].y = 0;
            vertices[segments].z = 0;
            vertices[segments].color = color.ToArgb();
            indices[2*segments - 1] = (short)(segments);
        }*/
    }
}
