using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using Math32.LinearAlgebra;
using System.Threading;

namespace Motr3D.Primitives
{    
    public abstract class Primitive: Object3D
    {
        private Color color = Color.YellowGreen;
        private volatile byte transparency = byte.MaxValue;
        protected volatile bool hasMouseOver = false, isMouseDown = false, pickable = true;
        protected volatile bool optimize = false;

        public event EventHandler ColorChanged;
        public event EventHandler PrimitiveChanged;

        public event MouseEventHandler Click;
        public event MouseEventHandler MouseUp;
        public event MouseEventHandler MouseDown;
        public event MouseEventHandler MouseEnter;
        public event MouseEventHandler MouseLeave;
        public event MouseEventHandler MouseOver;

        public virtual Color Color
        {
            get { return color; }
            set 
            {
                color = Color.FromArgb(transparency, value);
                OnColorChanged(); OnPrimitiveChanged(); 
            }
        }

        public virtual byte Transparency
        {
            get { return transparency; }
            set 
            {
                color = Color.FromArgb(value, color); 
                transparency = value; OnColorChanged(); OnPrimitiveChanged(); 
            }
        }

        public virtual bool Pickable { get { return pickable; } set { pickable = value; hasMouseOver = false; } }
       
        
        //protected override void DrawSelf(Camera camera) {  }
        
        public virtual void PickSelf(MouseEventArgs args, bool picked)
        {
            if (picked && pickable)
            {
                if (!hasMouseOver) { OnMouseEnter(args); hasMouseOver = true; }
                OnMouseOver(args);
                if (args.Button != MouseButtons.None) 
                {
                    if (!isMouseDown) { OnMouseDown(args); isMouseDown = true; }
                    OnClick(args);  
                }
            }
            if (hasMouseOver) { OnMouseLeave(args); hasMouseOver = false; }
            if (isMouseDown && (args.Button == MouseButtons.None)) { isMouseDown = false; OnMouseUp(args); }
        }
        public void OnColorChanged() { if (ColorChanged != null) { ColorChanged(this, null); } }
        public void OnPrimitiveChanged() { if (PrimitiveChanged != null) { PrimitiveChanged(this, null); } }
        public void OnClick(MouseEventArgs args) { if (Click != null) { Click(this, args); } }
        public void OnMouseUp(MouseEventArgs args) { if (MouseUp != null) { MouseUp(this, args); } }
        public void OnMouseDown(MouseEventArgs args) { if (MouseDown != null) { MouseDown(this, args); } }
        public void OnMouseEnter(MouseEventArgs args) { if (MouseEnter != null) { MouseEnter(this, args); } }
        public void OnMouseLeave(MouseEventArgs args) { if (MouseLeave != null) { MouseLeave(this, args); } }
        public void OnMouseOver(MouseEventArgs args) { if (MouseOver != null) { MouseOver(this, args); } }

    }
}
