using System;
using System.Collections.Generic;
using System.Text;
using Math32.LinearAlgebra;

namespace Motr3D.Primitives
{
    public class Surface : Mesh
    {
        protected float[][] x,y,z;
        
        public Surface(int lengthDivision,int depthDivision, float squareSize)
        {
            x = new float[lengthDivision][];
            y = new float[lengthDivision][];
            z = new float[lengthDivision][];
            for (int i = 0; i < lengthDivision; i++)
            { 
                x[i] = new float[depthDivision];
                y[i] = new float[depthDivision];
                z[i] = new float[depthDivision]; 
            }

            for (int i = 0; i < lengthDivision; i++) { for (int j = 0; j < lengthDivision; j++) { x[i][j] = squareSize *j; } }
            for (int i = 0; i < lengthDivision; i++) { for (int j = 0; j < lengthDivision; j++) { y[i][j] = 0.0f; } }
            for (int i = 0; i < lengthDivision; i++) { for (int j = 0; j < lengthDivision; j++) { z[j][i] = squareSize * j; } }
            GenerateVertexData();
        }
        
        //public Surface(float[] x, float[] y, float[] z) { GenerateSurface(x, y, z); }

        public float[][] X 
        { 
            get { return x; }
            set
            {
                x = value;
                if (value.Length == Vertices.Length)
                {
                    for (int i = 0; i < x.Rank; i++)
                    {
                        for (int j = 0; j < x.GetLength(0); j++) { Vertices[i * x.GetLength(0) + j].x = x[i][j]; } 
                    }
                }
                else { GenerateVertexData(); }   
            }
        }
        public float[][] Y
        {
            get { return y; }
            set
            {
                y = value;
                if (value.Length == Vertices.Length)
                {
                    for (int i = 0; i < y.Rank; i++)
                    {
                        for (int j = 0; j < y.GetLength(0); j++) { Vertices[i * z.GetLength(0) + j].y = y[i][j]; }
                    }
                }
                else { GenerateVertexData(); }
            }
        }
        public float[][] Z
        {
            get { return z; }
            set
            {
                z = value;
                if (value.Length == Vertices.Length)
                {
                    for (int i = 0; i < z.Rank; i++) 
                    {
                        for (int j = 0; j < z.GetLength(0); j++) { Vertices[i * z.GetLength(0) + j].z = z[i][j]; }
                    }
                }
                else { GenerateVertexData(); }
            }
        }
            
        public virtual void GenerateSurface(float[][] x, float[][] y, float[][] z)
        {
            this.x = x; this.y = y; this.z = z;
            GenerateVertexData();
        }

        public virtual void AppendToEnd(float[] x, float[] y, float[] z)
        {
        }

        public virtual void AppendToStart(float[] x, float[] y, float[] z)
        {

        }

        public virtual void AppendInside(float[] x, float[] y, float[] z, int index)
        {

        }

        protected int GetMinLengthAtIndex(int i)
        {
            if (x[i].Length <= y[i].Length && x[i].Length <= z[i].Length) { return x[i].Length; }
            else if (y[i].Length <= x[i].Length && y[i].Length <= z[i].Length) { return y[i].Length; }
            else if (z[i].Length <= x[i].Length && z[i].Length <= y[i].Length) { return z[i].Length; }
            else { return x[i].Length; }
        }

        protected override void GenerateVertexData()
        {
            int indiceIndex = 0;
            int maxI, maxJ = GetMinLengthAtIndex(0);
            int numVert = 0;

            if (x.GetLength(0) <= 1 || y.GetLength(0) <= 1 || z.GetLength(0) <= 1) { return; }
            if (x.GetLength(0) < y.GetLength(0) && x.GetLength(0) < z.GetLength(0)) { maxI = x.GetLength(0); }
            else if (y.GetLength(0) < x.GetLength(0) && y.GetLength(0) < z.GetLength(0)) { maxI = y.GetLength(0); }
            else if (z.GetLength(0) < y.GetLength(0) && z.GetLength(0) < x.GetLength(0)) { maxI = z.GetLength(0); }
            else { maxI = x.GetLength(0); }

            
            for (int j = 0; j < maxI; j++)
            {
                if (GetMinLengthAtIndex(j) < maxJ) { maxJ = GetMinLengthAtIndex(j); }
            }

            Vertex[] vertices = new Vertex[maxI * maxJ];
            short[] indices = new short[(maxI - 1) * (maxJ - 1) * 6];

            for (int i = 0; i < maxI; i++)
            {
                for (int j = 0; j < maxJ; j++) 
                { 
                    vertices[j+i*(maxJ)] = new Vertex(x[i][j], y[i][j], z[i][j], Color);
                    if (i < maxI - 1 && j < maxJ - 1)
                    {
                        indices[indiceIndex++] = (short)(j + i * (maxJ));
                        indices[indiceIndex++] = (short)(j + (i + 1) * (maxJ));
                        indices[indiceIndex++] = (short)(j + 1 + (i + 1) * (maxJ));
                        indices[indiceIndex++] = (short)(j + i * (maxJ));
                        indices[indiceIndex++] = (short)(j + 1 + (i + 1) * (maxJ));
                        indices[indiceIndex++] = (short)(j + 1 + i * (maxJ));
                    }
                }
            }
            Lock();
            Vertices = vertices;
            Indices = indices;
            Unlock();
            /*for(int i=0; i< maxJ; i++)
            {
                vertices[i + (maxI - 1) * maxJ] = new VertexStruct(x[maxI - 1][i], y[maxI - 1][i], z[maxI - 1][i], color);
            }*/
        }
        /*protected override void GenerateVertexData()
        {
            int indiceIndex = 0;
            int maxI, maxJ;


            if (x.GetLength(0) < y.GetLength(0) || x.GetLength(0) < z.GetLength(0)) { maxI = x.GetLength(0); }
            else if (y.GetLength(0) < x.GetLength(0) || y.GetLength(0) < z.GetLength(0)) { maxI = y.GetLength(0); }
            else if (z.GetLength(0) < y.GetLength(0) || z.GetLength(0) < x.GetLength(0)) { maxI = z.GetLength(0); }
            else { maxI = x.GetLength(0); }

            if (x.GetLength(1) < y.GetLength(1) || x.GetLength(1) < z.GetLength(1)) { maxJ = x.GetLength(1); }
            else if (y.GetLength(1) < x.GetLength(1) || y.GetLength(1) < z.GetLength(1)) { maxJ = y.GetLength(1); }
            else if (z.GetLength(1) < y.GetLength(1) || z.GetLength(1) < x.GetLength(1)) { maxJ = z.GetLength(1); }
            else { maxJ = x.GetLength(1); }

            vertices = new VertexStruct[maxI * maxJ];
            indices = new short[(maxI - 1) * (maxJ - 1) * 6];

            for (int i = 0; i < maxI; i++)
            {
                for (int j = 0; j < maxJ; j++) { vertices[j + i * maxJ] = new VertexStruct(x[i, j], y[i, j], z[i, j], color); }
            }

            for (int i = 0; i < maxI - 1; i++)
            {
                for (int j = 0; j < maxJ - 1; j++)
                {
                    indices[indiceIndex++] = (short)(j + i * (maxJ));
                    indices[indiceIndex++] = (short)(j + (i + 1) * (maxJ));
                    indices[indiceIndex++] = (short)(j + 1 + (i + 1) * (maxJ));
                    indices[indiceIndex++] = (short)(j + i * (maxJ));
                    indices[indiceIndex++] = (short)(j + 1 + (i + 1) * (maxJ));
                    indices[indiceIndex++] = (short)(j + 1 + i * (maxJ));
                }
            }

            base.GenerateVertexData();
        }*/
    }
}
