using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Threading;

namespace Motr3D.Primitives
{
    public abstract class Text: Primitive
    {
        protected string text = "";
        protected Font font;
        protected FontFamily fontFamily;
        protected float emSize = 8.0f;
        protected FontStyle fontStyle = FontStyle.Regular;

        public Text(string text) 
        { 
            this.text = text;
            fontFamily = new FontFamily("Arial");
            font = new Font(fontFamily, emSize);
        }

        public Text(string text, string fontName, float emSize)
        {
            this.text = text;
            fontFamily = new FontFamily(fontName);
            this.emSize = emSize;
            font = new Font(fontFamily, emSize);
        }

        public virtual FontFamily FontFamily { get { return fontFamily; } set { fontFamily = value; UpdateFont(); } }
        public virtual Font Font { get { return font; } set { font = value; OnPrimitiveChanged(); } }
        public virtual FontStyle FontStyle { get { return fontStyle; } set { fontStyle = value; UpdateFont(); } }
        public virtual string CurrentText
        { 
            get { return text; }
            set { Monitor.Enter(this); text = value; OnPrimitiveChanged(); Monitor.Exit(this); } 
        }
        public virtual float EmSize { get { return emSize; } set { emSize = value; UpdateFont(); } }

        protected override void DrawSelf(Camera camera)
        {
            camera.Viewport.Implementation.DrawObject(this);
        }

        protected virtual void UpdateFont() { Font = new Font(fontFamily, emSize, fontStyle); }
    }

    public class Text2D : Text 
    { 
        public Text2D(string text) : base(text) { }
        public Text2D(string text, string fontName, float emSize) : base(text, fontName, emSize) { }
    }

    public class Text3D : Text
    {
        protected float extrusion = 1.0f;
        public Text3D(string text) : base(text) { }
        public Text3D(string text, string fontName, float emSize, float extrusion) 
            : base(text, fontName, emSize)
        {
            this.extrusion = extrusion;
        }
        public virtual float Extrusion { get { return extrusion; } set { extrusion = value; OnPrimitiveChanged(); } }
    }
}
