using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Math32.LinearAlgebra;
using System.Threading;

namespace Motr3D.Primitives
{
    public abstract class VertexPrimitive: Primitive
    {
        private Vertex[] vertices = new Vertex[0];
        private short[] indices = new short[0];

        public event EventHandler VertexChanged;
        public event EventHandler IndiceChanged;

        public override Color Color
        {
            get { return base.Color; }
            set
            {
                base.Color = value;
                Monitor.Enter(this);
                for (uint i = 0; i < vertices.Length; i++)
                {
                    vertices[i].color = value.ToArgb() + ((int)Transparency << 24);
                }
                OnColorChanged();
                Monitor.Exit(this);
            }
        }

        public override byte Transparency
        {
            get { return base.Transparency; }
            set
            {
                Monitor.Enter(this);
                for (uint i = 0; i < vertices.Length; i++)
                {
                    vertices[i].color = Color.ToArgb() + ((int)value << 24);
                }
                Monitor.Exit(this);
                base.Transparency = value;
            }
        }

        public virtual Vertex[] Vertices
        {
            get { return vertices; }
            set
            {
                Monitor.Enter(this);
                vertices = value; OnVertexChanged(); OnPrimitiveChanged();
                Monitor.Exit(this);
            }
        }
        public virtual short[] Indices
        {
            get { return indices; }
            set { Monitor.Enter(this); indices = value; OnIndiceChanged(); OnPrimitiveChanged(); Monitor.Exit(this); }
        }
        public virtual bool Optimize { get { return optimize; } set { optimize = true; } }

        protected override void DrawSelf(Camera camera)
        {
            camera.Viewport.Implementation.DrawObject(this);
        }

        protected abstract void GenerateVertexData();

        public void OnVertexChanged() { if (VertexChanged != null) { VertexChanged(this, null); } }
        public void OnIndiceChanged() { if (IndiceChanged != null) { IndiceChanged(this, null); } }
    }
}
