using System;
using System.Collections.Generic;
using System.Text;
using Elements;
using SignalProcessing;
using Driver;
using URLServer;
using DataSetTypes;
using VirtualSignalGeneratorDriver;

namespace SimpleScope
{
    public class Channel: Element
    {
        private static uint channels = 0;

        private Trigger trigger = new Trigger("Trigger");
        private PerformanceFilter performanceFilter = new PerformanceFilter("PerformanceFilter");
        private uint channelNumber = channels++;
        private IPort channelOutputPort; 
        private URL sourceURL;
        private DataSet channelOutput;
        private RessourceHandler channelOutputResH, sourceResH;
        private List<URL> dataSources = new List<URL>();
        private int source = 0;
        private VirtualSignalGenerator virtualSignalGenerator = new VirtualSignalGenerator();

        public Channel() : base("Channel" + channels)
        {
            //URL[] ressources = LocalRessourceLocator.SearchRessource("Sample");
            dataSources.AddRange(virtualSignalGenerator.OutputPort.AvailableRessources);
            sourceURL.IsValid = false;
        }

       
        protected override void InitElement()
        {
            base.InitElement();
            channelOutputPort = new Port(this, "ChannelOutput");
            channelOutputResH = new AttributeRessourceHandler("ChannelOutput", channelOutputPort);
            trigger.SignalInputPort.Listen(performanceFilter.SignalOutputPort, "OutputSignal", "InputSignal");
            channelOutputPort.Listen(trigger.SignalOutputPort, "OutputSignal", "ChannelOutput");
            sourceResH = new AttributeRessourceHandler("Source", controlPort);
        }

        public override void NewRessources(URL[] newRessources)
        {
            base.NewRessources(newRessources);
            for(int i=0; i< newRessources.Length ;i++)
            {
                if (newRessources[i].Ressource == "Sample") { dataSources.Add(newRessources[i]); }
            }
        }
        public override void RessourcesLost(URL[] lostRessources)
        {
            base.RessourcesLost(lostRessources);
        }

        public DataSet ChannelOutput
        {
            get { return channelOutput; }
            set { channelOutput = value; channelOutputResH.Advertise(value); } 
        }
        public IPort ChannelOutputPort { get { return channelOutputPort; } }
        public uint Number { get { return channelNumber; } }
        public Trigger Trigger { get { return trigger; } }
        public PerformanceFilter PerformanceFilter { get { return performanceFilter; } }
        public URL SourceURL { get { return sourceURL; } }
        public List<URL> DataSources { get { return dataSources; } }
        public int Source
        { 
            get { return source; }
            set 
            { 
                if (sourceURL.IsValid)
                { 
                    performanceFilter.SignalInputPort.Quiet(sourceURL.Device, sourceURL.Ressource, "InputSignal");
                }
                source = value;
                if (source != 0)
                {
                    sourceURL = dataSources[value-1];
                    ((AbstractDriver)sourceURL.Device.Owner).Start();
                    performanceFilter.SignalInputPort.Listen(sourceURL.Device, sourceURL.Ressource, "InputSignal");
                }
                sourceResH.Advertise(value); 
            } 
        }

        protected override void DisposeElement()
        {
            base.DisposeElement();
            trigger.Dispose();
            performanceFilter.Dispose();
            channelOutputPort.Dispose();
            if (channelNumber == channels) { channels--; }
        }
    }
}
