using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Elements;
using Driver;
using SoundCardInputDriver;
using SignalProcessing.GUI;
using SoundCardInputDriver.GUI;
using VirtualSignalGeneratorDriver.GUI;
using VirtualSignalGeneratorDriver;

namespace SimpleScope
{
    public partial class ChannelGUI: UserControl
    {
        private Channel owner;
        protected SoundInputChannel currentInputChannel;
        protected DriverControl currentInputChannelGUI;

        private PerformanceFilterGUI performanceFilterGUI;
        protected TriggerGUI triggerGUI;

        private ChannelGUI() { InitializeComponent(); }
        public ChannelGUI(Channel owner) 
        {
            InitializeComponent();
            this.owner = owner;
            channelGroupBox.Text = "Channel " + owner.Number;
            performanceFilterGUI = new PerformanceFilterGUI(owner.PerformanceFilter);
            channelLayoutPanel.Controls.Add(performanceFilterGUI, 2, 0);
            triggerGUI = new TriggerGUI(owner.Trigger);
            channelLayoutPanel.Controls.Add(triggerGUI, 2, 1);
            sourceListBox.SelectedIndex = 0;
            foreach (URL url in owner.DataSources) { sourceListBox.Items.Add(url.Module); }
        }
        
        private void sourceListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SuspendLayout();

            if (currentInputChannelGUI != null)
            {
                channelLayoutPanel.Controls.Remove(currentInputChannelGUI);
                currentInputChannelGUI.Dispose();
            }
            if (sourceListBox.SelectedIndex == 0) { return; }

            owner.Source = sourceListBox.SelectedIndex;

            if (sourceListBox.SelectedIndex == 1)
            {
                currentInputChannelGUI = new VirtualSignalGeneratorGUI((VirtualSignalGenerator)owner.SourceURL.Device.Owner);
            }
            else
            {
                currentInputChannelGUI = new SoundInputChannelGUI((SoundInputChannel)owner.SourceURL.Device.Owner);
            }

            channelLayoutPanel.Controls.Add(currentInputChannelGUI, 1, 0);
            channelLayoutPanel.SetRowSpan(currentInputChannelGUI, 2);
            this.ResumeLayout();
        }
    }
}
