using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DataSetVisualizer;
using SignalProcessing;
using Motr3D;
using Motr3D.D3DImplementation;
using XYZDataSetVisualizer.GUI;
using XYZDataSetVisualizer;

namespace SimpleScope
{
    public partial class Display : Form
    {
        private static uint maxDisplayNumber = 0;
        
        protected class DataSetViewControlPair 
        { 
            public XYZDataSetView view; 
            public XYZDataSetViewGUI control;
            public Button button = new Button();
        }
        private Visualizer visualizer;
        private List<DataSetViewControlPair> dataSetViewControlPairs = new List<DataSetViewControlPair>();
        protected int currentNumber = 0;
        private uint displayNumber = maxDisplayNumber++;

        public Display()
        {
            InitializeComponent();
            Text = "Display " + displayNumber;
            visualizer = new Visualizer(new Motr3DD3DImplementation());
            splitContainer.Panel2.Controls.Add(visualizer);
            visualizer.Dock = DockStyle.Fill;
            addNewToolStripMenuItem_Click(null, null);
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            if (MessageBox.Show(
                this, 
                "Are you sure you want to close this display ?",
                "Confirmation",
                MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                Dispose();
            }
            else
            {
                e.Cancel = true;
            }
            base.OnClosing(e);
        }

        private void addNewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DataSetViewControlPair dsvcp = new DataSetViewControlPair();
            
            splitContainer.SuspendLayout();
            
            dsvcp.view = new XYZDataSetView("Signal_" + currentNumber);
            dsvcp.view.LocalRessourceLocator = Elements.Element.DomainRessourceLocator;
            dsvcp.control = new XYZDataSetViewGUI("Signal " + currentNumber, dsvcp.view);
            visualizer.AddSignal(dsvcp.view);
            dataSetViewControlPairs.Add(dsvcp);
            channelControlLayoutPanel.Controls.Add(dsvcp.control);
            dsvcp.button.Text = dsvcp.view.ElementName;
            ToolStripMenuItem t = new ToolStripMenuItem("Signal "+ currentNumber++);
            t.Tag = dsvcp;
            t.Click += new EventHandler(removeDataSetControl);
            removeToolStripMenuItem.DropDownItems.Add(t);
            channelLabelLayoutPanel.Controls.Add(dsvcp.button);
            dsvcp.button.Tag = dsvcp;
            dsvcp.button.Click += new EventHandler(channelButtonClicked);
            splitContainer.ResumeLayout();
            dsvcp.view.OnFocused();
        }

        protected void removeDataSetControl(object sender, EventArgs e)
        {
            splitContainer.SuspendLayout();
            DataSetViewControlPair dsvcp = ((ToolStripMenuItem)sender).Tag as DataSetViewControlPair;
            if (dataSetViewControlPairs.IndexOf(dsvcp) == dataSetViewControlPairs.Count - 1)
            { 
                currentNumber--; 
            }
            dataSetViewControlPairs.Remove(dsvcp);
            dsvcp.view.Dispose();
            dsvcp.control.Dispose();
            channelLabelLayoutPanel.Controls.Remove(dsvcp.button);
            removeToolStripMenuItem.DropDownItems.Remove((ToolStripMenuItem)sender);
            splitContainer.ResumeLayout();
        }

        protected virtual void channelButtonClicked(object sender, EventArgs args)
        {
            DataSetViewControlPair dsvcp = ((Button)sender).Tag as DataSetViewControlPair;
            dsvcp.view.OnFocused();
        }

        private void expandButton_Click(object sender, EventArgs e)
        {
            if (splitContainer.Panel1Collapsed) { splitContainer.Panel1Collapsed = false; expandButton.Text = "E"; }
            else { splitContainer.Panel1Collapsed = true; expandButton.Text = "C"; }
        }
    }
}