using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SoundCardInputDriver;
using URLServer;
using Driver;
using Elements;

namespace SimpleScope
{
    public partial class SimpleScope : Form
    {
        protected class ChannelGUIPair { public Channel channel; public ChannelGUI channelGUI;}

        protected List<ChannelGUIPair> channels = new List<ChannelGUIPair>();
        protected List<Display> visualizers = new List<Display>();
        protected Enumerator soundInputEnumerator;
        private Motr3D.Demo.Motr3DDemo easterEgg;

        public SimpleScope()
        { 
            InitializeComponent();
            Element.DomainRessourceLocator = new Server("URLServer");
            soundInputEnumerator = new SoundInputEnumerator();
            foreach(SoundInputChannel ad in soundInputEnumerator.Drivers)
            {
                Element.DomainRessourceLocator.RegisterURLs(ad.SamplePort.AvailableRessources);
            }
            addNewChannelToolStripMenuItem_Click(null, null);
            displaysToolStripMenuItem_Click(null, null);
        }
        
        private void addNewChannelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ChannelGUIPair c = new ChannelGUIPair();
            c.channel = new Channel();
            c.channel.LocalRessourceLocator = Element.DomainRessourceLocator;
            c.channelGUI = new ChannelGUI(c.channel);
            channels.Add(c);
            channelLayoutPanel.Controls.Add(c.channelGUI);
            ToolStripMenuItem t = new ToolStripMenuItem("Channel " + c.channel.Number);
            t.Click += new EventHandler(removeClickedChannel);
            t.Tag = c;
            removeChannelToolStripMenuItem.DropDownItems.Add(t);
            Element.DomainRessourceLocator.RegisterURLs(c.channel.ChannelOutputPort.AvailableRessources);
        }

        private void removeClickedChannel(object sender, EventArgs e)
        {
            ChannelGUIPair c = (ChannelGUIPair)((ToolStripMenuItem)sender).Tag;
            Element.DomainRessourceLocator.UnRegisterURLs(c.channel.ChannelOutputPort.AvailableRessources);
            c.channel.Dispose();
            c.channelGUI.Dispose();
            channels.Remove(c);
            ((ToolStripMenuItem)sender).Dispose();
        }

        private void DisplayFormClosed(object sender, FormClosedEventArgs e) { visualizers.Remove((Display)sender); }

        [STAThread]
        public static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new SimpleScope());
            Application.Exit();
        }

        private void displaysToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Display v = new Display();
            visualizers.Add(v);
            v.Show();
            v.FormClosed += new FormClosedEventHandler(DisplayFormClosed);
        }

        private void motr3DDemoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (easterEgg == null)
            {
                easterEgg = new Motr3D.Demo.Motr3DDemo(new Motr3D.D3DImplementation.Motr3DD3DImplementation());
                easterEgg.Show();
                easterEgg.FormClosed += new FormClosedEventHandler(motr3DDemoClosed);
            }
        }
        private void motr3DDemoClosed(object sender, FormClosedEventArgs e)
        {
            easterEgg = null;
        }
    }
}